# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

from enum import Enum

__all__ = [
    'CacheMode',
    'CompressionType',
    'Platform',
]


class CacheMode(str, Enum):
    MIN = "min"
    """
    Only layers that are exported into the resulting image are cached.
    """
    MAX = "max"
    """
    All layers are cached, even those of intermediate steps.
    """


class CompressionType(str, Enum):
    GZIP = "gzip"
    """
    Use `gzip` for compression.
    """
    ESTARGZ = "estargz"
    """
    Use `estargz` for compression.
    """
    ZSTD = "zstd"
    """
    Use `zstd` for compression.
    """


class Platform(str, Enum):
    DARWIN_386 = "darwin/386"
    DARWIN_AMD64 = "darwin/amd64"
    DARWIN_ARM = "darwin/arm"
    DARWIN_ARM64 = "darwin/arm64"
    DRAGONFLY_AMD64 = "dragonfly/amd64"
    FREEBSD_386 = "freebsd/386"
    FREEBSD_AMD64 = "freebsd/amd64"
    FREEBSD_ARM = "freebsd/arm"
    LINUX_386 = "linux/386"
    LINUX_AMD64 = "linux/amd64"
    LINUX_ARM = "linux/arm"
    LINUX_ARM64 = "linux/arm64"
    LINUX_MIPS64 = "linux/mips64"
    LINUX_MIPS64LE = "linux/mips64le"
    LINUX_PPC64LE = "linux/ppc64le"
    LINUX_RISCV64 = "linux/riscv64"
    LINUX_S390X = "linux/s390x"
    NETBSD_386 = "netbsd/386"
    NETBSD_AMD64 = "netbsd/amd64"
    NETBSD_ARM = "netbsd/arm"
    OPENBSD_386 = "openbsd/386"
    OPENBSD_AMD64 = "openbsd/amd64"
    OPENBSD_ARM = "openbsd/arm"
    PLAN9_386 = "plan9/386"
    PLAN9_AMD64 = "plan9/amd64"
    SOLARIS_AMD64 = "solaris/amd64"
    WINDOWS_386 = "windows/386"
    WINDOWS_AMD64 = "windows/amd64"



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from .. import _utilities
from . import outputs
from ._enums import *

__all__ = [
    'BuildContext',
    'BuilderConfig',
    'CacheFromAzureBlob',
    'CacheFromEntry',
    'CacheFromGitHubActions',
    'CacheFromLocal',
    'CacheFromRegistry',
    'CacheFromS3',
    'CacheInline',
    'CacheToAzureBlob',
    'CacheToEntry',
    'CacheToGitHubActions',
    'CacheToLocal',
    'CacheToRegistry',
    'CacheToS3',
    'Context',
    'Dockerfile',
    'ExportDocker',
    'ExportEntry',
    'ExportImage',
    'ExportLocal',
    'ExportOCI',
    'ExportRegistry',
    'ExportTar',
    'Manifest',
    'ManifestPlatform',
    'RegistryAuth',
]

@pulumi.output_type
class BuildContext(dict):
    def __init__(__self__, *,
                 location: str,
                 named: Optional[Mapping[str, 'outputs.Context']] = None):
        """
        :param str location: Resources to use for build context.
               
               The location can be:
               * A relative or absolute path to a local directory (`.`, `./app`,
                 `/app`, etc.).
               * A remote URL of a Git repository, tarball, or plain text file
                 (`https://github.com/user/myrepo.git`, `http://server/context.tar.gz`,
                 etc.).
        :param Mapping[str, 'ContextArgs'] named: Additional build contexts to use. 
               
               These contexts are accessed with `FROM name` or `--from=name`
               statements when using Dockerfile 1.4+ syntax.
               
               Values can be local paths, HTTP URLs, or  `docker-image://` images.
        """
        pulumi.set(__self__, "location", location)
        if named is not None:
            pulumi.set(__self__, "named", named)

    @property
    @pulumi.getter
    def location(self) -> str:
        """
        Resources to use for build context.

        The location can be:
        * A relative or absolute path to a local directory (`.`, `./app`,
          `/app`, etc.).
        * A remote URL of a Git repository, tarball, or plain text file
          (`https://github.com/user/myrepo.git`, `http://server/context.tar.gz`,
          etc.).
        """
        return pulumi.get(self, "location")

    @property
    @pulumi.getter
    def named(self) -> Optional[Mapping[str, 'outputs.Context']]:
        """
        Additional build contexts to use. 

        These contexts are accessed with `FROM name` or `--from=name`
        statements when using Dockerfile 1.4+ syntax.

        Values can be local paths, HTTP URLs, or  `docker-image://` images.
        """
        return pulumi.get(self, "named")


@pulumi.output_type
class BuilderConfig(dict):
    def __init__(__self__, *,
                 name: Optional[str] = None):
        """
        :param str name: Name of an existing buildx builder to use.
               
               Only `docker-container`, `kubernetes`, or `remote` drivers are
               supported. The legacy `docker` driver is not supported.
        """
        if name is not None:
            pulumi.set(__self__, "name", name)

    @property
    @pulumi.getter
    def name(self) -> Optional[str]:
        """
        Name of an existing buildx builder to use.

        Only `docker-container`, `kubernetes`, or `remote` drivers are
        supported. The legacy `docker` driver is not supported.
        """
        return pulumi.get(self, "name")


@pulumi.output_type
class CacheFromAzureBlob(dict):
    @staticmethod
    def __key_warning(key: str):
        suggest = None
        if key == "accountUrl":
            suggest = "account_url"
        elif key == "secretAccessKey":
            suggest = "secret_access_key"

        if suggest:
            pulumi.log.warn(f"Key '{key}' not found in CacheFromAzureBlob. Access the value via the '{suggest}' property getter instead.")

    def __getitem__(self, key: str) -> Any:
        CacheFromAzureBlob.__key_warning(key)
        return super().__getitem__(key)

    def get(self, key: str, default = None) -> Any:
        CacheFromAzureBlob.__key_warning(key)
        return super().get(key, default)

    def __init__(__self__, *,
                 name: str,
                 account_url: Optional[str] = None,
                 secret_access_key: Optional[str] = None):
        """
        :param str name: The name of the cache image.
        :param str account_url: Base URL of the storage account.
        :param str secret_access_key: Blob storage account key.
        """
        pulumi.set(__self__, "name", name)
        if account_url is not None:
            pulumi.set(__self__, "account_url", account_url)
        if secret_access_key is not None:
            pulumi.set(__self__, "secret_access_key", secret_access_key)

    @property
    @pulumi.getter
    def name(self) -> str:
        """
        The name of the cache image.
        """
        return pulumi.get(self, "name")

    @property
    @pulumi.getter(name="accountUrl")
    def account_url(self) -> Optional[str]:
        """
        Base URL of the storage account.
        """
        return pulumi.get(self, "account_url")

    @property
    @pulumi.getter(name="secretAccessKey")
    def secret_access_key(self) -> Optional[str]:
        """
        Blob storage account key.
        """
        return pulumi.get(self, "secret_access_key")


@pulumi.output_type
class CacheFromEntry(dict):
    def __init__(__self__, *,
                 azblob: Optional['outputs.CacheFromAzureBlob'] = None,
                 gha: Optional['outputs.CacheFromGitHubActions'] = None,
                 inline: Optional['outputs.CacheInline'] = None,
                 local: Optional['outputs.CacheFromLocal'] = None,
                 raw: Optional[str] = None,
                 registry: Optional['outputs.CacheFromRegistry'] = None,
                 s3: Optional['outputs.CacheFromS3'] = None):
        """
        :param 'CacheFromAzureBlobArgs' azblob: Upload build caches to Azure's blob storage service.
        :param 'CacheFromGitHubActionsArgs' gha: Recommended for use with GitHub Actions workflows.
               
               An action like `crazy-max/ghaction-github-runtime` is recommended to
               expose appropriate credentials to your GitHub workflow.
        :param 'CacheInlineArgs' inline: The inline cache storage backend is the simplest implementation to get
               started with, but it does not handle multi-stage builds. Consider the
               registry cache backend instead.
        :param 'CacheFromLocalArgs' local: A simple backend which caches images on your local filesystem.
        :param str raw: A raw string as you would provide it to the Docker CLI (e.g.,
               `type=inline`).
        :param 'CacheFromRegistryArgs' registry: Upload build caches to remote registries.
        :param 'CacheFromS3Args' s3: Upload build caches to AWS S3 or an S3-compatible services such as
               MinIO.
        """
        if azblob is not None:
            pulumi.set(__self__, "azblob", azblob)
        if gha is not None:
            pulumi.set(__self__, "gha", gha)
        if inline is not None:
            pulumi.set(__self__, "inline", inline)
        if local is not None:
            pulumi.set(__self__, "local", local)
        if raw is not None:
            pulumi.set(__self__, "raw", raw)
        if registry is not None:
            pulumi.set(__self__, "registry", registry)
        if s3 is not None:
            pulumi.set(__self__, "s3", s3)

    @property
    @pulumi.getter
    def azblob(self) -> Optional['outputs.CacheFromAzureBlob']:
        """
        Upload build caches to Azure's blob storage service.
        """
        return pulumi.get(self, "azblob")

    @property
    @pulumi.getter
    def gha(self) -> Optional['outputs.CacheFromGitHubActions']:
        """
        Recommended for use with GitHub Actions workflows.

        An action like `crazy-max/ghaction-github-runtime` is recommended to
        expose appropriate credentials to your GitHub workflow.
        """
        return pulumi.get(self, "gha")

    @property
    @pulumi.getter
    def inline(self) -> Optional['outputs.CacheInline']:
        """
        The inline cache storage backend is the simplest implementation to get
        started with, but it does not handle multi-stage builds. Consider the
        registry cache backend instead.
        """
        return pulumi.get(self, "inline")

    @property
    @pulumi.getter
    def local(self) -> Optional['outputs.CacheFromLocal']:
        """
        A simple backend which caches images on your local filesystem.
        """
        return pulumi.get(self, "local")

    @property
    @pulumi.getter
    def raw(self) -> Optional[str]:
        """
        A raw string as you would provide it to the Docker CLI (e.g.,
        `type=inline`).
        """
        return pulumi.get(self, "raw")

    @property
    @pulumi.getter
    def registry(self) -> Optional['outputs.CacheFromRegistry']:
        """
        Upload build caches to remote registries.
        """
        return pulumi.get(self, "registry")

    @property
    @pulumi.getter
    def s3(self) -> Optional['outputs.CacheFromS3']:
        """
        Upload build caches to AWS S3 or an S3-compatible services such as
        MinIO.
        """
        return pulumi.get(self, "s3")


@pulumi.output_type
class CacheFromGitHubActions(dict):
    def __init__(__self__, *,
                 scope: Optional[str] = None,
                 token: Optional[str] = None,
                 url: Optional[str] = None):
        """
        :param str scope: The scope to use for cache keys. Defaults to `buildkit`.
               
               This should be set if building and caching multiple images in one
               workflow, otherwise caches will overwrite each other.
        :param str token: The GitHub Actions token to use. This is not a personal access tokens
               and is typically generated automatically as part of each job.
               
               Defaults to `$ACTIONS_RUNTIME_TOKEN`, although a separate action like
               `crazy-max/ghaction-github-runtime` is recommended to expose this
               environment variable to your jobs.
        :param str url: The cache server URL to use for artifacts.
               
               Defaults to `$ACTIONS_RUNTIME_URL`, although a separate action like
               `crazy-max/ghaction-github-runtime` is recommended to expose this
               environment variable to your jobs.
        """
        if scope is None:
            scope = (_utilities.get_env('buildkit') or '')
        if scope is not None:
            pulumi.set(__self__, "scope", scope)
        if token is None:
            token = (_utilities.get_env('ACTIONS_RUNTIME_TOKEN') or '')
        if token is not None:
            pulumi.set(__self__, "token", token)
        if url is None:
            url = (_utilities.get_env('ACTIONS_RUNTIME_URL') or '')
        if url is not None:
            pulumi.set(__self__, "url", url)

    @property
    @pulumi.getter
    def scope(self) -> Optional[str]:
        """
        The scope to use for cache keys. Defaults to `buildkit`.

        This should be set if building and caching multiple images in one
        workflow, otherwise caches will overwrite each other.
        """
        return pulumi.get(self, "scope")

    @property
    @pulumi.getter
    def token(self) -> Optional[str]:
        """
        The GitHub Actions token to use. This is not a personal access tokens
        and is typically generated automatically as part of each job.

        Defaults to `$ACTIONS_RUNTIME_TOKEN`, although a separate action like
        `crazy-max/ghaction-github-runtime` is recommended to expose this
        environment variable to your jobs.
        """
        return pulumi.get(self, "token")

    @property
    @pulumi.getter
    def url(self) -> Optional[str]:
        """
        The cache server URL to use for artifacts.

        Defaults to `$ACTIONS_RUNTIME_URL`, although a separate action like
        `crazy-max/ghaction-github-runtime` is recommended to expose this
        environment variable to your jobs.
        """
        return pulumi.get(self, "url")


@pulumi.output_type
class CacheFromLocal(dict):
    def __init__(__self__, *,
                 src: str,
                 digest: Optional[str] = None):
        """
        :param str src: Path of the local directory where cache gets imported from.
        :param str digest: Digest of manifest to import.
        """
        pulumi.set(__self__, "src", src)
        if digest is not None:
            pulumi.set(__self__, "digest", digest)

    @property
    @pulumi.getter
    def src(self) -> str:
        """
        Path of the local directory where cache gets imported from.
        """
        return pulumi.get(self, "src")

    @property
    @pulumi.getter
    def digest(self) -> Optional[str]:
        """
        Digest of manifest to import.
        """
        return pulumi.get(self, "digest")


@pulumi.output_type
class CacheFromRegistry(dict):
    def __init__(__self__, *,
                 ref: str):
        """
        :param str ref: Fully qualified name of the cache image to import.
        """
        pulumi.set(__self__, "ref", ref)

    @property
    @pulumi.getter
    def ref(self) -> str:
        """
        Fully qualified name of the cache image to import.
        """
        return pulumi.get(self, "ref")


@pulumi.output_type
class CacheFromS3(dict):
    @staticmethod
    def __key_warning(key: str):
        suggest = None
        if key == "accessKeyId":
            suggest = "access_key_id"
        elif key == "blobsPrefix":
            suggest = "blobs_prefix"
        elif key == "endpointUrl":
            suggest = "endpoint_url"
        elif key == "manifestsPrefix":
            suggest = "manifests_prefix"
        elif key == "secretAccessKey":
            suggest = "secret_access_key"
        elif key == "sessionToken":
            suggest = "session_token"
        elif key == "usePathStyle":
            suggest = "use_path_style"

        if suggest:
            pulumi.log.warn(f"Key '{key}' not found in CacheFromS3. Access the value via the '{suggest}' property getter instead.")

    def __getitem__(self, key: str) -> Any:
        CacheFromS3.__key_warning(key)
        return super().__getitem__(key)

    def get(self, key: str, default = None) -> Any:
        CacheFromS3.__key_warning(key)
        return super().get(key, default)

    def __init__(__self__, *,
                 bucket: str,
                 region: Optional[str] = None,
                 access_key_id: Optional[str] = None,
                 blobs_prefix: Optional[str] = None,
                 endpoint_url: Optional[str] = None,
                 manifests_prefix: Optional[str] = None,
                 name: Optional[str] = None,
                 secret_access_key: Optional[str] = None,
                 session_token: Optional[str] = None,
                 use_path_style: Optional[bool] = None):
        """
        :param str bucket: Name of the S3 bucket.
        :param str region: The geographic location of the bucket. Defaults to `$AWS_REGION`.
        :param str access_key_id: Defaults to `$AWS_ACCESS_KEY_ID`.
        :param str blobs_prefix: Prefix to prepend to blob filenames.
        :param str endpoint_url: Endpoint of the S3 bucket.
        :param str manifests_prefix: Prefix to prepend on manifest filenames.
        :param str name: Name of the cache image.
        :param str secret_access_key: Defaults to `$AWS_SECRET_ACCESS_KEY`.
        :param str session_token: Defaults to `$AWS_SESSION_TOKEN`.
        :param bool use_path_style: Uses `bucket` in the URL instead of hostname when `true`.
        """
        pulumi.set(__self__, "bucket", bucket)
        if region is None:
            region = (_utilities.get_env('AWS_REGION') or '')
        pulumi.set(__self__, "region", region)
        if access_key_id is None:
            access_key_id = (_utilities.get_env('AWS_ACCESS_KEY_ID') or '')
        if access_key_id is not None:
            pulumi.set(__self__, "access_key_id", access_key_id)
        if blobs_prefix is not None:
            pulumi.set(__self__, "blobs_prefix", blobs_prefix)
        if endpoint_url is not None:
            pulumi.set(__self__, "endpoint_url", endpoint_url)
        if manifests_prefix is not None:
            pulumi.set(__self__, "manifests_prefix", manifests_prefix)
        if name is not None:
            pulumi.set(__self__, "name", name)
        if secret_access_key is None:
            secret_access_key = (_utilities.get_env('AWS_SECRET_ACCESS_KEY') or '')
        if secret_access_key is not None:
            pulumi.set(__self__, "secret_access_key", secret_access_key)
        if session_token is None:
            session_token = (_utilities.get_env('AWS_SESSION_TOKEN') or '')
        if session_token is not None:
            pulumi.set(__self__, "session_token", session_token)
        if use_path_style is not None:
            pulumi.set(__self__, "use_path_style", use_path_style)

    @property
    @pulumi.getter
    def bucket(self) -> str:
        """
        Name of the S3 bucket.
        """
        return pulumi.get(self, "bucket")

    @property
    @pulumi.getter
    def region(self) -> str:
        """
        The geographic location of the bucket. Defaults to `$AWS_REGION`.
        """
        return pulumi.get(self, "region")

    @property
    @pulumi.getter(name="accessKeyId")
    def access_key_id(self) -> Optional[str]:
        """
        Defaults to `$AWS_ACCESS_KEY_ID`.
        """
        return pulumi.get(self, "access_key_id")

    @property
    @pulumi.getter(name="blobsPrefix")
    def blobs_prefix(self) -> Optional[str]:
        """
        Prefix to prepend to blob filenames.
        """
        return pulumi.get(self, "blobs_prefix")

    @property
    @pulumi.getter(name="endpointUrl")
    def endpoint_url(self) -> Optional[str]:
        """
        Endpoint of the S3 bucket.
        """
        return pulumi.get(self, "endpoint_url")

    @property
    @pulumi.getter(name="manifestsPrefix")
    def manifests_prefix(self) -> Optional[str]:
        """
        Prefix to prepend on manifest filenames.
        """
        return pulumi.get(self, "manifests_prefix")

    @property
    @pulumi.getter
    def name(self) -> Optional[str]:
        """
        Name of the cache image.
        """
        return pulumi.get(self, "name")

    @property
    @pulumi.getter(name="secretAccessKey")
    def secret_access_key(self) -> Optional[str]:
        """
        Defaults to `$AWS_SECRET_ACCESS_KEY`.
        """
        return pulumi.get(self, "secret_access_key")

    @property
    @pulumi.getter(name="sessionToken")
    def session_token(self) -> Optional[str]:
        """
        Defaults to `$AWS_SESSION_TOKEN`.
        """
        return pulumi.get(self, "session_token")

    @property
    @pulumi.getter(name="usePathStyle")
    def use_path_style(self) -> Optional[bool]:
        """
        Uses `bucket` in the URL instead of hostname when `true`.
        """
        return pulumi.get(self, "use_path_style")


@pulumi.output_type
class CacheInline(dict):
    def __init__(__self__):
        pass


@pulumi.output_type
class CacheToAzureBlob(dict):
    @staticmethod
    def __key_warning(key: str):
        suggest = None
        if key == "accountUrl":
            suggest = "account_url"
        elif key == "ignoreError":
            suggest = "ignore_error"
        elif key == "secretAccessKey":
            suggest = "secret_access_key"

        if suggest:
            pulumi.log.warn(f"Key '{key}' not found in CacheToAzureBlob. Access the value via the '{suggest}' property getter instead.")

    def __getitem__(self, key: str) -> Any:
        CacheToAzureBlob.__key_warning(key)
        return super().__getitem__(key)

    def get(self, key: str, default = None) -> Any:
        CacheToAzureBlob.__key_warning(key)
        return super().get(key, default)

    def __init__(__self__, *,
                 name: str,
                 account_url: Optional[str] = None,
                 ignore_error: Optional[bool] = None,
                 mode: Optional['CacheMode'] = None,
                 secret_access_key: Optional[str] = None):
        """
        :param str name: The name of the cache image.
        :param str account_url: Base URL of the storage account.
        :param bool ignore_error: Ignore errors caused by failed cache exports.
        :param 'CacheMode' mode: The cache mode to use. Defaults to `min`.
        :param str secret_access_key: Blob storage account key.
        """
        pulumi.set(__self__, "name", name)
        if account_url is not None:
            pulumi.set(__self__, "account_url", account_url)
        if ignore_error is None:
            ignore_error = False
        if ignore_error is not None:
            pulumi.set(__self__, "ignore_error", ignore_error)
        if mode is None:
            mode = 'min'
        if mode is not None:
            pulumi.set(__self__, "mode", mode)
        if secret_access_key is not None:
            pulumi.set(__self__, "secret_access_key", secret_access_key)

    @property
    @pulumi.getter
    def name(self) -> str:
        """
        The name of the cache image.
        """
        return pulumi.get(self, "name")

    @property
    @pulumi.getter(name="accountUrl")
    def account_url(self) -> Optional[str]:
        """
        Base URL of the storage account.
        """
        return pulumi.get(self, "account_url")

    @property
    @pulumi.getter(name="ignoreError")
    def ignore_error(self) -> Optional[bool]:
        """
        Ignore errors caused by failed cache exports.
        """
        return pulumi.get(self, "ignore_error")

    @property
    @pulumi.getter
    def mode(self) -> Optional['CacheMode']:
        """
        The cache mode to use. Defaults to `min`.
        """
        return pulumi.get(self, "mode")

    @property
    @pulumi.getter(name="secretAccessKey")
    def secret_access_key(self) -> Optional[str]:
        """
        Blob storage account key.
        """
        return pulumi.get(self, "secret_access_key")


@pulumi.output_type
class CacheToEntry(dict):
    def __init__(__self__, *,
                 azblob: Optional['outputs.CacheToAzureBlob'] = None,
                 gha: Optional['outputs.CacheToGitHubActions'] = None,
                 inline: Optional['outputs.CacheInline'] = None,
                 local: Optional['outputs.CacheToLocal'] = None,
                 raw: Optional[str] = None,
                 registry: Optional['outputs.CacheToRegistry'] = None,
                 s3: Optional['outputs.CacheToS3'] = None):
        """
        :param 'CacheToAzureBlobArgs' azblob: Push cache to Azure's blob storage service.
        :param 'CacheToGitHubActionsArgs' gha: Recommended for use with GitHub Actions workflows.
               
               An action like `crazy-max/ghaction-github-runtime` is recommended to
               expose appropriate credentials to your GitHub workflow.
        :param 'CacheInlineArgs' inline: The inline cache storage backend is the simplest implementation to get
               started with, but it does not handle multi-stage builds. Consider the
               `registry` cache backend instead.
        :param 'CacheToLocalArgs' local: A simple backend which caches imagines on your local filesystem.
        :param str raw: A raw string as you would provide it to the Docker CLI (e.g.,
               `type=inline`)
        :param 'CacheToRegistryArgs' registry: Push caches to remote registries. Incompatible with the `docker` build
               driver.
        :param 'CacheToS3Args' s3: Push cache to AWS S3 or S3-compatible services such as MinIO.
        """
        if azblob is not None:
            pulumi.set(__self__, "azblob", azblob)
        if gha is not None:
            pulumi.set(__self__, "gha", gha)
        if inline is not None:
            pulumi.set(__self__, "inline", inline)
        if local is not None:
            pulumi.set(__self__, "local", local)
        if raw is not None:
            pulumi.set(__self__, "raw", raw)
        if registry is not None:
            pulumi.set(__self__, "registry", registry)
        if s3 is not None:
            pulumi.set(__self__, "s3", s3)

    @property
    @pulumi.getter
    def azblob(self) -> Optional['outputs.CacheToAzureBlob']:
        """
        Push cache to Azure's blob storage service.
        """
        return pulumi.get(self, "azblob")

    @property
    @pulumi.getter
    def gha(self) -> Optional['outputs.CacheToGitHubActions']:
        """
        Recommended for use with GitHub Actions workflows.

        An action like `crazy-max/ghaction-github-runtime` is recommended to
        expose appropriate credentials to your GitHub workflow.
        """
        return pulumi.get(self, "gha")

    @property
    @pulumi.getter
    def inline(self) -> Optional['outputs.CacheInline']:
        """
        The inline cache storage backend is the simplest implementation to get
        started with, but it does not handle multi-stage builds. Consider the
        `registry` cache backend instead.
        """
        return pulumi.get(self, "inline")

    @property
    @pulumi.getter
    def local(self) -> Optional['outputs.CacheToLocal']:
        """
        A simple backend which caches imagines on your local filesystem.
        """
        return pulumi.get(self, "local")

    @property
    @pulumi.getter
    def raw(self) -> Optional[str]:
        """
        A raw string as you would provide it to the Docker CLI (e.g.,
        `type=inline`)
        """
        return pulumi.get(self, "raw")

    @property
    @pulumi.getter
    def registry(self) -> Optional['outputs.CacheToRegistry']:
        """
        Push caches to remote registries. Incompatible with the `docker` build
        driver.
        """
        return pulumi.get(self, "registry")

    @property
    @pulumi.getter
    def s3(self) -> Optional['outputs.CacheToS3']:
        """
        Push cache to AWS S3 or S3-compatible services such as MinIO.
        """
        return pulumi.get(self, "s3")


@pulumi.output_type
class CacheToGitHubActions(dict):
    @staticmethod
    def __key_warning(key: str):
        suggest = None
        if key == "ignoreError":
            suggest = "ignore_error"

        if suggest:
            pulumi.log.warn(f"Key '{key}' not found in CacheToGitHubActions. Access the value via the '{suggest}' property getter instead.")

    def __getitem__(self, key: str) -> Any:
        CacheToGitHubActions.__key_warning(key)
        return super().__getitem__(key)

    def get(self, key: str, default = None) -> Any:
        CacheToGitHubActions.__key_warning(key)
        return super().get(key, default)

    def __init__(__self__, *,
                 ignore_error: Optional[bool] = None,
                 mode: Optional['CacheMode'] = None,
                 scope: Optional[str] = None,
                 token: Optional[str] = None,
                 url: Optional[str] = None):
        """
        :param bool ignore_error: Ignore errors caused by failed cache exports.
        :param 'CacheMode' mode: The cache mode to use. Defaults to `min`.
        :param str scope: The scope to use for cache keys. Defaults to `buildkit`.
               
               This should be set if building and caching multiple images in one
               workflow, otherwise caches will overwrite each other.
        :param str token: The GitHub Actions token to use. This is not a personal access tokens
               and is typically generated automatically as part of each job.
               
               Defaults to `$ACTIONS_RUNTIME_TOKEN`, although a separate action like
               `crazy-max/ghaction-github-runtime` is recommended to expose this
               environment variable to your jobs.
        :param str url: The cache server URL to use for artifacts.
               
               Defaults to `$ACTIONS_RUNTIME_URL`, although a separate action like
               `crazy-max/ghaction-github-runtime` is recommended to expose this
               environment variable to your jobs.
        """
        if ignore_error is None:
            ignore_error = False
        if ignore_error is not None:
            pulumi.set(__self__, "ignore_error", ignore_error)
        if mode is None:
            mode = 'min'
        if mode is not None:
            pulumi.set(__self__, "mode", mode)
        if scope is None:
            scope = (_utilities.get_env('buildkit') or '')
        if scope is not None:
            pulumi.set(__self__, "scope", scope)
        if token is None:
            token = (_utilities.get_env('ACTIONS_RUNTIME_TOKEN') or '')
        if token is not None:
            pulumi.set(__self__, "token", token)
        if url is None:
            url = (_utilities.get_env('ACTIONS_RUNTIME_URL') or '')
        if url is not None:
            pulumi.set(__self__, "url", url)

    @property
    @pulumi.getter(name="ignoreError")
    def ignore_error(self) -> Optional[bool]:
        """
        Ignore errors caused by failed cache exports.
        """
        return pulumi.get(self, "ignore_error")

    @property
    @pulumi.getter
    def mode(self) -> Optional['CacheMode']:
        """
        The cache mode to use. Defaults to `min`.
        """
        return pulumi.get(self, "mode")

    @property
    @pulumi.getter
    def scope(self) -> Optional[str]:
        """
        The scope to use for cache keys. Defaults to `buildkit`.

        This should be set if building and caching multiple images in one
        workflow, otherwise caches will overwrite each other.
        """
        return pulumi.get(self, "scope")

    @property
    @pulumi.getter
    def token(self) -> Optional[str]:
        """
        The GitHub Actions token to use. This is not a personal access tokens
        and is typically generated automatically as part of each job.

        Defaults to `$ACTIONS_RUNTIME_TOKEN`, although a separate action like
        `crazy-max/ghaction-github-runtime` is recommended to expose this
        environment variable to your jobs.
        """
        return pulumi.get(self, "token")

    @property
    @pulumi.getter
    def url(self) -> Optional[str]:
        """
        The cache server URL to use for artifacts.

        Defaults to `$ACTIONS_RUNTIME_URL`, although a separate action like
        `crazy-max/ghaction-github-runtime` is recommended to expose this
        environment variable to your jobs.
        """
        return pulumi.get(self, "url")


@pulumi.output_type
class CacheToLocal(dict):
    @staticmethod
    def __key_warning(key: str):
        suggest = None
        if key == "compressionLevel":
            suggest = "compression_level"
        elif key == "forceCompression":
            suggest = "force_compression"
        elif key == "ignoreError":
            suggest = "ignore_error"

        if suggest:
            pulumi.log.warn(f"Key '{key}' not found in CacheToLocal. Access the value via the '{suggest}' property getter instead.")

    def __getitem__(self, key: str) -> Any:
        CacheToLocal.__key_warning(key)
        return super().__getitem__(key)

    def get(self, key: str, default = None) -> Any:
        CacheToLocal.__key_warning(key)
        return super().get(key, default)

    def __init__(__self__, *,
                 dest: str,
                 compression: Optional['CompressionType'] = None,
                 compression_level: Optional[int] = None,
                 force_compression: Optional[bool] = None,
                 ignore_error: Optional[bool] = None,
                 mode: Optional['CacheMode'] = None):
        """
        :param str dest: Path of the local directory to export the cache.
        :param 'CompressionType' compression: The compression type to use.
        :param int compression_level: Compression level from 0 to 22.
        :param bool force_compression: Forcefully apply compression.
        :param bool ignore_error: Ignore errors caused by failed cache exports.
        :param 'CacheMode' mode: The cache mode to use. Defaults to `min`.
        """
        pulumi.set(__self__, "dest", dest)
        if compression is None:
            compression = 'gzip'
        if compression is not None:
            pulumi.set(__self__, "compression", compression)
        if compression_level is None:
            compression_level = 0
        if compression_level is not None:
            pulumi.set(__self__, "compression_level", compression_level)
        if force_compression is None:
            force_compression = False
        if force_compression is not None:
            pulumi.set(__self__, "force_compression", force_compression)
        if ignore_error is None:
            ignore_error = False
        if ignore_error is not None:
            pulumi.set(__self__, "ignore_error", ignore_error)
        if mode is None:
            mode = 'min'
        if mode is not None:
            pulumi.set(__self__, "mode", mode)

    @property
    @pulumi.getter
    def dest(self) -> str:
        """
        Path of the local directory to export the cache.
        """
        return pulumi.get(self, "dest")

    @property
    @pulumi.getter
    def compression(self) -> Optional['CompressionType']:
        """
        The compression type to use.
        """
        return pulumi.get(self, "compression")

    @property
    @pulumi.getter(name="compressionLevel")
    def compression_level(self) -> Optional[int]:
        """
        Compression level from 0 to 22.
        """
        return pulumi.get(self, "compression_level")

    @property
    @pulumi.getter(name="forceCompression")
    def force_compression(self) -> Optional[bool]:
        """
        Forcefully apply compression.
        """
        return pulumi.get(self, "force_compression")

    @property
    @pulumi.getter(name="ignoreError")
    def ignore_error(self) -> Optional[bool]:
        """
        Ignore errors caused by failed cache exports.
        """
        return pulumi.get(self, "ignore_error")

    @property
    @pulumi.getter
    def mode(self) -> Optional['CacheMode']:
        """
        The cache mode to use. Defaults to `min`.
        """
        return pulumi.get(self, "mode")


@pulumi.output_type
class CacheToRegistry(dict):
    @staticmethod
    def __key_warning(key: str):
        suggest = None
        if key == "compressionLevel":
            suggest = "compression_level"
        elif key == "forceCompression":
            suggest = "force_compression"
        elif key == "ignoreError":
            suggest = "ignore_error"
        elif key == "imageManifest":
            suggest = "image_manifest"
        elif key == "ociMediaTypes":
            suggest = "oci_media_types"

        if suggest:
            pulumi.log.warn(f"Key '{key}' not found in CacheToRegistry. Access the value via the '{suggest}' property getter instead.")

    def __getitem__(self, key: str) -> Any:
        CacheToRegistry.__key_warning(key)
        return super().__getitem__(key)

    def get(self, key: str, default = None) -> Any:
        CacheToRegistry.__key_warning(key)
        return super().get(key, default)

    def __init__(__self__, *,
                 ref: str,
                 compression: Optional['CompressionType'] = None,
                 compression_level: Optional[int] = None,
                 force_compression: Optional[bool] = None,
                 ignore_error: Optional[bool] = None,
                 image_manifest: Optional[bool] = None,
                 mode: Optional['CacheMode'] = None,
                 oci_media_types: Optional[bool] = None):
        """
        :param str ref: Fully qualified name of the cache image to import.
        :param 'CompressionType' compression: The compression type to use.
        :param int compression_level: Compression level from 0 to 22.
        :param bool force_compression: Forcefully apply compression.
        :param bool ignore_error: Ignore errors caused by failed cache exports.
        :param bool image_manifest: Export cache manifest as an OCI-compatible image manifest instead of a
               manifest list (requires OCI media types).
               
               Defaults to `false`.
        :param 'CacheMode' mode: The cache mode to use. Defaults to `min`.
        :param bool oci_media_types: Whether to use OCI mediatypes in exported manifests. Defaults to
               `true`.
        """
        pulumi.set(__self__, "ref", ref)
        if compression is None:
            compression = 'gzip'
        if compression is not None:
            pulumi.set(__self__, "compression", compression)
        if compression_level is None:
            compression_level = 0
        if compression_level is not None:
            pulumi.set(__self__, "compression_level", compression_level)
        if force_compression is None:
            force_compression = False
        if force_compression is not None:
            pulumi.set(__self__, "force_compression", force_compression)
        if ignore_error is None:
            ignore_error = False
        if ignore_error is not None:
            pulumi.set(__self__, "ignore_error", ignore_error)
        if image_manifest is None:
            image_manifest = False
        if image_manifest is not None:
            pulumi.set(__self__, "image_manifest", image_manifest)
        if mode is None:
            mode = 'min'
        if mode is not None:
            pulumi.set(__self__, "mode", mode)
        if oci_media_types is None:
            oci_media_types = True
        if oci_media_types is not None:
            pulumi.set(__self__, "oci_media_types", oci_media_types)

    @property
    @pulumi.getter
    def ref(self) -> str:
        """
        Fully qualified name of the cache image to import.
        """
        return pulumi.get(self, "ref")

    @property
    @pulumi.getter
    def compression(self) -> Optional['CompressionType']:
        """
        The compression type to use.
        """
        return pulumi.get(self, "compression")

    @property
    @pulumi.getter(name="compressionLevel")
    def compression_level(self) -> Optional[int]:
        """
        Compression level from 0 to 22.
        """
        return pulumi.get(self, "compression_level")

    @property
    @pulumi.getter(name="forceCompression")
    def force_compression(self) -> Optional[bool]:
        """
        Forcefully apply compression.
        """
        return pulumi.get(self, "force_compression")

    @property
    @pulumi.getter(name="ignoreError")
    def ignore_error(self) -> Optional[bool]:
        """
        Ignore errors caused by failed cache exports.
        """
        return pulumi.get(self, "ignore_error")

    @property
    @pulumi.getter(name="imageManifest")
    def image_manifest(self) -> Optional[bool]:
        """
        Export cache manifest as an OCI-compatible image manifest instead of a
        manifest list (requires OCI media types).

        Defaults to `false`.
        """
        return pulumi.get(self, "image_manifest")

    @property
    @pulumi.getter
    def mode(self) -> Optional['CacheMode']:
        """
        The cache mode to use. Defaults to `min`.
        """
        return pulumi.get(self, "mode")

    @property
    @pulumi.getter(name="ociMediaTypes")
    def oci_media_types(self) -> Optional[bool]:
        """
        Whether to use OCI mediatypes in exported manifests. Defaults to
        `true`.
        """
        return pulumi.get(self, "oci_media_types")


@pulumi.output_type
class CacheToS3(dict):
    @staticmethod
    def __key_warning(key: str):
        suggest = None
        if key == "accessKeyId":
            suggest = "access_key_id"
        elif key == "blobsPrefix":
            suggest = "blobs_prefix"
        elif key == "endpointUrl":
            suggest = "endpoint_url"
        elif key == "ignoreError":
            suggest = "ignore_error"
        elif key == "manifestsPrefix":
            suggest = "manifests_prefix"
        elif key == "secretAccessKey":
            suggest = "secret_access_key"
        elif key == "sessionToken":
            suggest = "session_token"
        elif key == "usePathStyle":
            suggest = "use_path_style"

        if suggest:
            pulumi.log.warn(f"Key '{key}' not found in CacheToS3. Access the value via the '{suggest}' property getter instead.")

    def __getitem__(self, key: str) -> Any:
        CacheToS3.__key_warning(key)
        return super().__getitem__(key)

    def get(self, key: str, default = None) -> Any:
        CacheToS3.__key_warning(key)
        return super().get(key, default)

    def __init__(__self__, *,
                 bucket: str,
                 region: Optional[str] = None,
                 access_key_id: Optional[str] = None,
                 blobs_prefix: Optional[str] = None,
                 endpoint_url: Optional[str] = None,
                 ignore_error: Optional[bool] = None,
                 manifests_prefix: Optional[str] = None,
                 mode: Optional['CacheMode'] = None,
                 name: Optional[str] = None,
                 secret_access_key: Optional[str] = None,
                 session_token: Optional[str] = None,
                 use_path_style: Optional[bool] = None):
        """
        :param str bucket: Name of the S3 bucket.
        :param str region: The geographic location of the bucket. Defaults to `$AWS_REGION`.
        :param str access_key_id: Defaults to `$AWS_ACCESS_KEY_ID`.
        :param str blobs_prefix: Prefix to prepend to blob filenames.
        :param str endpoint_url: Endpoint of the S3 bucket.
        :param bool ignore_error: Ignore errors caused by failed cache exports.
        :param str manifests_prefix: Prefix to prepend on manifest filenames.
        :param 'CacheMode' mode: The cache mode to use. Defaults to `min`.
        :param str name: Name of the cache image.
        :param str secret_access_key: Defaults to `$AWS_SECRET_ACCESS_KEY`.
        :param str session_token: Defaults to `$AWS_SESSION_TOKEN`.
        :param bool use_path_style: Uses `bucket` in the URL instead of hostname when `true`.
        """
        pulumi.set(__self__, "bucket", bucket)
        if region is None:
            region = (_utilities.get_env('AWS_REGION') or '')
        pulumi.set(__self__, "region", region)
        if access_key_id is None:
            access_key_id = (_utilities.get_env('AWS_ACCESS_KEY_ID') or '')
        if access_key_id is not None:
            pulumi.set(__self__, "access_key_id", access_key_id)
        if blobs_prefix is not None:
            pulumi.set(__self__, "blobs_prefix", blobs_prefix)
        if endpoint_url is not None:
            pulumi.set(__self__, "endpoint_url", endpoint_url)
        if ignore_error is None:
            ignore_error = False
        if ignore_error is not None:
            pulumi.set(__self__, "ignore_error", ignore_error)
        if manifests_prefix is not None:
            pulumi.set(__self__, "manifests_prefix", manifests_prefix)
        if mode is None:
            mode = 'min'
        if mode is not None:
            pulumi.set(__self__, "mode", mode)
        if name is not None:
            pulumi.set(__self__, "name", name)
        if secret_access_key is None:
            secret_access_key = (_utilities.get_env('AWS_SECRET_ACCESS_KEY') or '')
        if secret_access_key is not None:
            pulumi.set(__self__, "secret_access_key", secret_access_key)
        if session_token is None:
            session_token = (_utilities.get_env('AWS_SESSION_TOKEN') or '')
        if session_token is not None:
            pulumi.set(__self__, "session_token", session_token)
        if use_path_style is not None:
            pulumi.set(__self__, "use_path_style", use_path_style)

    @property
    @pulumi.getter
    def bucket(self) -> str:
        """
        Name of the S3 bucket.
        """
        return pulumi.get(self, "bucket")

    @property
    @pulumi.getter
    def region(self) -> str:
        """
        The geographic location of the bucket. Defaults to `$AWS_REGION`.
        """
        return pulumi.get(self, "region")

    @property
    @pulumi.getter(name="accessKeyId")
    def access_key_id(self) -> Optional[str]:
        """
        Defaults to `$AWS_ACCESS_KEY_ID`.
        """
        return pulumi.get(self, "access_key_id")

    @property
    @pulumi.getter(name="blobsPrefix")
    def blobs_prefix(self) -> Optional[str]:
        """
        Prefix to prepend to blob filenames.
        """
        return pulumi.get(self, "blobs_prefix")

    @property
    @pulumi.getter(name="endpointUrl")
    def endpoint_url(self) -> Optional[str]:
        """
        Endpoint of the S3 bucket.
        """
        return pulumi.get(self, "endpoint_url")

    @property
    @pulumi.getter(name="ignoreError")
    def ignore_error(self) -> Optional[bool]:
        """
        Ignore errors caused by failed cache exports.
        """
        return pulumi.get(self, "ignore_error")

    @property
    @pulumi.getter(name="manifestsPrefix")
    def manifests_prefix(self) -> Optional[str]:
        """
        Prefix to prepend on manifest filenames.
        """
        return pulumi.get(self, "manifests_prefix")

    @property
    @pulumi.getter
    def mode(self) -> Optional['CacheMode']:
        """
        The cache mode to use. Defaults to `min`.
        """
        return pulumi.get(self, "mode")

    @property
    @pulumi.getter
    def name(self) -> Optional[str]:
        """
        Name of the cache image.
        """
        return pulumi.get(self, "name")

    @property
    @pulumi.getter(name="secretAccessKey")
    def secret_access_key(self) -> Optional[str]:
        """
        Defaults to `$AWS_SECRET_ACCESS_KEY`.
        """
        return pulumi.get(self, "secret_access_key")

    @property
    @pulumi.getter(name="sessionToken")
    def session_token(self) -> Optional[str]:
        """
        Defaults to `$AWS_SESSION_TOKEN`.
        """
        return pulumi.get(self, "session_token")

    @property
    @pulumi.getter(name="usePathStyle")
    def use_path_style(self) -> Optional[bool]:
        """
        Uses `bucket` in the URL instead of hostname when `true`.
        """
        return pulumi.get(self, "use_path_style")


@pulumi.output_type
class Context(dict):
    def __init__(__self__, *,
                 location: str):
        """
        :param str location: Resources to use for build context.
               
               The location can be:
               * A relative or absolute path to a local directory (`.`, `./app`,
                 `/app`, etc.).
               * A remote URL of a Git repository, tarball, or plain text file
                 (`https://github.com/user/myrepo.git`, `http://server/context.tar.gz`,
                 etc.).
        """
        pulumi.set(__self__, "location", location)

    @property
    @pulumi.getter
    def location(self) -> str:
        """
        Resources to use for build context.

        The location can be:
        * A relative or absolute path to a local directory (`.`, `./app`,
          `/app`, etc.).
        * A remote URL of a Git repository, tarball, or plain text file
          (`https://github.com/user/myrepo.git`, `http://server/context.tar.gz`,
          etc.).
        """
        return pulumi.get(self, "location")


@pulumi.output_type
class Dockerfile(dict):
    def __init__(__self__, *,
                 inline: Optional[str] = None,
                 location: Optional[str] = None):
        """
        :param str inline: Raw Dockerfile contents.
               
               Conflicts with `location`.
        :param str location: Location of the Dockerfile to use.
               
               Can be a relative or absolute path to a local file, or a remote URL.
               
               Conflicts with `inline`.
        """
        if inline is not None:
            pulumi.set(__self__, "inline", inline)
        if location is not None:
            pulumi.set(__self__, "location", location)

    @property
    @pulumi.getter
    def inline(self) -> Optional[str]:
        """
        Raw Dockerfile contents.

        Conflicts with `location`.
        """
        return pulumi.get(self, "inline")

    @property
    @pulumi.getter
    def location(self) -> Optional[str]:
        """
        Location of the Dockerfile to use.

        Can be a relative or absolute path to a local file, or a remote URL.

        Conflicts with `inline`.
        """
        return pulumi.get(self, "location")


@pulumi.output_type
class ExportDocker(dict):
    @staticmethod
    def __key_warning(key: str):
        suggest = None
        if key == "compressionLevel":
            suggest = "compression_level"
        elif key == "forceCompression":
            suggest = "force_compression"
        elif key == "ociMediaTypes":
            suggest = "oci_media_types"

        if suggest:
            pulumi.log.warn(f"Key '{key}' not found in ExportDocker. Access the value via the '{suggest}' property getter instead.")

    def __getitem__(self, key: str) -> Any:
        ExportDocker.__key_warning(key)
        return super().__getitem__(key)

    def get(self, key: str, default = None) -> Any:
        ExportDocker.__key_warning(key)
        return super().get(key, default)

    def __init__(__self__, *,
                 annotations: Optional[Mapping[str, str]] = None,
                 compression: Optional['CompressionType'] = None,
                 compression_level: Optional[int] = None,
                 dest: Optional[str] = None,
                 force_compression: Optional[bool] = None,
                 names: Optional[Sequence[str]] = None,
                 oci_media_types: Optional[bool] = None,
                 tar: Optional[bool] = None):
        """
        :param Mapping[str, str] annotations: Attach an arbitrary key/value annotation to the image.
        :param 'CompressionType' compression: The compression type to use.
        :param int compression_level: Compression level from 0 to 22.
        :param str dest: The local export path.
        :param bool force_compression: Forcefully apply compression.
        :param Sequence[str] names: Specify images names to export. This is overridden if tags are already specified.
        :param bool oci_media_types: Use OCI media types in exporter manifests.
        :param bool tar: Bundle the output into a tarball layout.
        """
        if annotations is not None:
            pulumi.set(__self__, "annotations", annotations)
        if compression is None:
            compression = 'gzip'
        if compression is not None:
            pulumi.set(__self__, "compression", compression)
        if compression_level is None:
            compression_level = 0
        if compression_level is not None:
            pulumi.set(__self__, "compression_level", compression_level)
        if dest is not None:
            pulumi.set(__self__, "dest", dest)
        if force_compression is None:
            force_compression = False
        if force_compression is not None:
            pulumi.set(__self__, "force_compression", force_compression)
        if names is not None:
            pulumi.set(__self__, "names", names)
        if oci_media_types is None:
            oci_media_types = False
        if oci_media_types is not None:
            pulumi.set(__self__, "oci_media_types", oci_media_types)
        if tar is None:
            tar = True
        if tar is not None:
            pulumi.set(__self__, "tar", tar)

    @property
    @pulumi.getter
    def annotations(self) -> Optional[Mapping[str, str]]:
        """
        Attach an arbitrary key/value annotation to the image.
        """
        return pulumi.get(self, "annotations")

    @property
    @pulumi.getter
    def compression(self) -> Optional['CompressionType']:
        """
        The compression type to use.
        """
        return pulumi.get(self, "compression")

    @property
    @pulumi.getter(name="compressionLevel")
    def compression_level(self) -> Optional[int]:
        """
        Compression level from 0 to 22.
        """
        return pulumi.get(self, "compression_level")

    @property
    @pulumi.getter
    def dest(self) -> Optional[str]:
        """
        The local export path.
        """
        return pulumi.get(self, "dest")

    @property
    @pulumi.getter(name="forceCompression")
    def force_compression(self) -> Optional[bool]:
        """
        Forcefully apply compression.
        """
        return pulumi.get(self, "force_compression")

    @property
    @pulumi.getter
    def names(self) -> Optional[Sequence[str]]:
        """
        Specify images names to export. This is overridden if tags are already specified.
        """
        return pulumi.get(self, "names")

    @property
    @pulumi.getter(name="ociMediaTypes")
    def oci_media_types(self) -> Optional[bool]:
        """
        Use OCI media types in exporter manifests.
        """
        return pulumi.get(self, "oci_media_types")

    @property
    @pulumi.getter
    def tar(self) -> Optional[bool]:
        """
        Bundle the output into a tarball layout.
        """
        return pulumi.get(self, "tar")


@pulumi.output_type
class ExportEntry(dict):
    def __init__(__self__, *,
                 docker: Optional['outputs.ExportDocker'] = None,
                 image: Optional['outputs.ExportImage'] = None,
                 local: Optional['outputs.ExportLocal'] = None,
                 manifests: Optional[Sequence['outputs.Manifest']] = None,
                 oci: Optional['outputs.ExportOCI'] = None,
                 raw: Optional[str] = None,
                 registry: Optional['outputs.ExportRegistry'] = None,
                 tar: Optional['outputs.ExportTar'] = None):
        """
        :param 'ExportDockerArgs' docker: Export as a Docker image layout.
        :param 'ExportImageArgs' image: Outputs the build result into a container image format.
        :param 'ExportLocalArgs' local: Export to a local directory as files and directories.
        :param Sequence['ManifestArgs'] manifests: An output property populated for exporters that pushed image
               manifest(s) to a registry.
        :param 'ExportOCIArgs' oci: Identical to the Docker exporter but uses OCI media types by default.
        :param str raw: A raw string as you would provide it to the Docker CLI (e.g.,
               `type=docker`)
        :param 'ExportRegistryArgs' registry: Identical to the Image exporter, but pushes by default.
        :param 'ExportTarArgs' tar: Export to a local directory as a tarball.
        """
        if docker is not None:
            pulumi.set(__self__, "docker", docker)
        if image is not None:
            pulumi.set(__self__, "image", image)
        if local is not None:
            pulumi.set(__self__, "local", local)
        if manifests is not None:
            pulumi.set(__self__, "manifests", manifests)
        if oci is not None:
            pulumi.set(__self__, "oci", oci)
        if raw is not None:
            pulumi.set(__self__, "raw", raw)
        if registry is not None:
            pulumi.set(__self__, "registry", registry)
        if tar is not None:
            pulumi.set(__self__, "tar", tar)

    @property
    @pulumi.getter
    def docker(self) -> Optional['outputs.ExportDocker']:
        """
        Export as a Docker image layout.
        """
        return pulumi.get(self, "docker")

    @property
    @pulumi.getter
    def image(self) -> Optional['outputs.ExportImage']:
        """
        Outputs the build result into a container image format.
        """
        return pulumi.get(self, "image")

    @property
    @pulumi.getter
    def local(self) -> Optional['outputs.ExportLocal']:
        """
        Export to a local directory as files and directories.
        """
        return pulumi.get(self, "local")

    @property
    @pulumi.getter
    def manifests(self) -> Optional[Sequence['outputs.Manifest']]:
        """
        An output property populated for exporters that pushed image
        manifest(s) to a registry.
        """
        return pulumi.get(self, "manifests")

    @property
    @pulumi.getter
    def oci(self) -> Optional['outputs.ExportOCI']:
        """
        Identical to the Docker exporter but uses OCI media types by default.
        """
        return pulumi.get(self, "oci")

    @property
    @pulumi.getter
    def raw(self) -> Optional[str]:
        """
        A raw string as you would provide it to the Docker CLI (e.g.,
        `type=docker`)
        """
        return pulumi.get(self, "raw")

    @property
    @pulumi.getter
    def registry(self) -> Optional['outputs.ExportRegistry']:
        """
        Identical to the Image exporter, but pushes by default.
        """
        return pulumi.get(self, "registry")

    @property
    @pulumi.getter
    def tar(self) -> Optional['outputs.ExportTar']:
        """
        Export to a local directory as a tarball.
        """
        return pulumi.get(self, "tar")


@pulumi.output_type
class ExportImage(dict):
    @staticmethod
    def __key_warning(key: str):
        suggest = None
        if key == "compressionLevel":
            suggest = "compression_level"
        elif key == "danglingNamePrefix":
            suggest = "dangling_name_prefix"
        elif key == "forceCompression":
            suggest = "force_compression"
        elif key == "nameCanonical":
            suggest = "name_canonical"
        elif key == "ociMediaTypes":
            suggest = "oci_media_types"
        elif key == "pushByDigest":
            suggest = "push_by_digest"

        if suggest:
            pulumi.log.warn(f"Key '{key}' not found in ExportImage. Access the value via the '{suggest}' property getter instead.")

    def __getitem__(self, key: str) -> Any:
        ExportImage.__key_warning(key)
        return super().__getitem__(key)

    def get(self, key: str, default = None) -> Any:
        ExportImage.__key_warning(key)
        return super().get(key, default)

    def __init__(__self__, *,
                 annotations: Optional[Mapping[str, str]] = None,
                 compression: Optional['CompressionType'] = None,
                 compression_level: Optional[int] = None,
                 dangling_name_prefix: Optional[str] = None,
                 force_compression: Optional[bool] = None,
                 insecure: Optional[bool] = None,
                 name_canonical: Optional[bool] = None,
                 names: Optional[Sequence[str]] = None,
                 oci_media_types: Optional[bool] = None,
                 push: Optional[bool] = None,
                 push_by_digest: Optional[bool] = None,
                 store: Optional[bool] = None,
                 unpack: Optional[bool] = None):
        """
        :param Mapping[str, str] annotations: Attach an arbitrary key/value annotation to the image.
        :param 'CompressionType' compression: The compression type to use.
        :param int compression_level: Compression level from 0 to 22.
        :param str dangling_name_prefix: Name image with `prefix@<digest>`, used for anonymous images.
        :param bool force_compression: Forcefully apply compression.
        :param bool insecure: Allow pushing to an insecure registry.
        :param bool name_canonical: Add additional canonical name (`name@<digest>`).
        :param Sequence[str] names: Specify images names to export. This is overridden if tags are already specified.
        :param bool oci_media_types: Use OCI media types in exporter manifests.
        :param bool push: Push after creating the image.
        :param bool push_by_digest: Push image without name.
        :param bool store: Store resulting images to the worker's image store and ensure all of
               its blobs are in the content store. 
               
               Defaults to `true`.
               
               Ignored if the worker doesn't have image store (when using OCI workers,
               for example).
        :param bool unpack: Unpack image after creation (for use with containerd). Defaults to
               `false`.
        """
        if annotations is not None:
            pulumi.set(__self__, "annotations", annotations)
        if compression is None:
            compression = 'gzip'
        if compression is not None:
            pulumi.set(__self__, "compression", compression)
        if compression_level is None:
            compression_level = 0
        if compression_level is not None:
            pulumi.set(__self__, "compression_level", compression_level)
        if dangling_name_prefix is not None:
            pulumi.set(__self__, "dangling_name_prefix", dangling_name_prefix)
        if force_compression is None:
            force_compression = False
        if force_compression is not None:
            pulumi.set(__self__, "force_compression", force_compression)
        if insecure is not None:
            pulumi.set(__self__, "insecure", insecure)
        if name_canonical is not None:
            pulumi.set(__self__, "name_canonical", name_canonical)
        if names is not None:
            pulumi.set(__self__, "names", names)
        if oci_media_types is None:
            oci_media_types = False
        if oci_media_types is not None:
            pulumi.set(__self__, "oci_media_types", oci_media_types)
        if push is not None:
            pulumi.set(__self__, "push", push)
        if push_by_digest is not None:
            pulumi.set(__self__, "push_by_digest", push_by_digest)
        if store is None:
            store = True
        if store is not None:
            pulumi.set(__self__, "store", store)
        if unpack is not None:
            pulumi.set(__self__, "unpack", unpack)

    @property
    @pulumi.getter
    def annotations(self) -> Optional[Mapping[str, str]]:
        """
        Attach an arbitrary key/value annotation to the image.
        """
        return pulumi.get(self, "annotations")

    @property
    @pulumi.getter
    def compression(self) -> Optional['CompressionType']:
        """
        The compression type to use.
        """
        return pulumi.get(self, "compression")

    @property
    @pulumi.getter(name="compressionLevel")
    def compression_level(self) -> Optional[int]:
        """
        Compression level from 0 to 22.
        """
        return pulumi.get(self, "compression_level")

    @property
    @pulumi.getter(name="danglingNamePrefix")
    def dangling_name_prefix(self) -> Optional[str]:
        """
        Name image with `prefix@<digest>`, used for anonymous images.
        """
        return pulumi.get(self, "dangling_name_prefix")

    @property
    @pulumi.getter(name="forceCompression")
    def force_compression(self) -> Optional[bool]:
        """
        Forcefully apply compression.
        """
        return pulumi.get(self, "force_compression")

    @property
    @pulumi.getter
    def insecure(self) -> Optional[bool]:
        """
        Allow pushing to an insecure registry.
        """
        return pulumi.get(self, "insecure")

    @property
    @pulumi.getter(name="nameCanonical")
    def name_canonical(self) -> Optional[bool]:
        """
        Add additional canonical name (`name@<digest>`).
        """
        return pulumi.get(self, "name_canonical")

    @property
    @pulumi.getter
    def names(self) -> Optional[Sequence[str]]:
        """
        Specify images names to export. This is overridden if tags are already specified.
        """
        return pulumi.get(self, "names")

    @property
    @pulumi.getter(name="ociMediaTypes")
    def oci_media_types(self) -> Optional[bool]:
        """
        Use OCI media types in exporter manifests.
        """
        return pulumi.get(self, "oci_media_types")

    @property
    @pulumi.getter
    def push(self) -> Optional[bool]:
        """
        Push after creating the image.
        """
        return pulumi.get(self, "push")

    @property
    @pulumi.getter(name="pushByDigest")
    def push_by_digest(self) -> Optional[bool]:
        """
        Push image without name.
        """
        return pulumi.get(self, "push_by_digest")

    @property
    @pulumi.getter
    def store(self) -> Optional[bool]:
        """
        Store resulting images to the worker's image store and ensure all of
        its blobs are in the content store. 

        Defaults to `true`.

        Ignored if the worker doesn't have image store (when using OCI workers,
        for example).
        """
        return pulumi.get(self, "store")

    @property
    @pulumi.getter
    def unpack(self) -> Optional[bool]:
        """
        Unpack image after creation (for use with containerd). Defaults to
        `false`.
        """
        return pulumi.get(self, "unpack")


@pulumi.output_type
class ExportLocal(dict):
    def __init__(__self__, *,
                 dest: str):
        """
        :param str dest: Output path.
        """
        pulumi.set(__self__, "dest", dest)

    @property
    @pulumi.getter
    def dest(self) -> str:
        """
        Output path.
        """
        return pulumi.get(self, "dest")


@pulumi.output_type
class ExportOCI(dict):
    @staticmethod
    def __key_warning(key: str):
        suggest = None
        if key == "compressionLevel":
            suggest = "compression_level"
        elif key == "forceCompression":
            suggest = "force_compression"
        elif key == "ociMediaTypes":
            suggest = "oci_media_types"

        if suggest:
            pulumi.log.warn(f"Key '{key}' not found in ExportOCI. Access the value via the '{suggest}' property getter instead.")

    def __getitem__(self, key: str) -> Any:
        ExportOCI.__key_warning(key)
        return super().__getitem__(key)

    def get(self, key: str, default = None) -> Any:
        ExportOCI.__key_warning(key)
        return super().get(key, default)

    def __init__(__self__, *,
                 annotations: Optional[Mapping[str, str]] = None,
                 compression: Optional['CompressionType'] = None,
                 compression_level: Optional[int] = None,
                 dest: Optional[str] = None,
                 force_compression: Optional[bool] = None,
                 names: Optional[Sequence[str]] = None,
                 oci_media_types: Optional[bool] = None,
                 tar: Optional[bool] = None):
        """
        :param Mapping[str, str] annotations: Attach an arbitrary key/value annotation to the image.
        :param 'CompressionType' compression: The compression type to use.
        :param int compression_level: Compression level from 0 to 22.
        :param str dest: The local export path.
        :param bool force_compression: Forcefully apply compression.
        :param Sequence[str] names: Specify images names to export. This is overridden if tags are already specified.
        :param bool oci_media_types: Use OCI media types in exporter manifests.
        :param bool tar: Bundle the output into a tarball layout.
        """
        if annotations is not None:
            pulumi.set(__self__, "annotations", annotations)
        if compression is None:
            compression = 'gzip'
        if compression is not None:
            pulumi.set(__self__, "compression", compression)
        if compression_level is None:
            compression_level = 0
        if compression_level is not None:
            pulumi.set(__self__, "compression_level", compression_level)
        if dest is not None:
            pulumi.set(__self__, "dest", dest)
        if force_compression is None:
            force_compression = False
        if force_compression is not None:
            pulumi.set(__self__, "force_compression", force_compression)
        if names is not None:
            pulumi.set(__self__, "names", names)
        if oci_media_types is None:
            oci_media_types = True
        if oci_media_types is not None:
            pulumi.set(__self__, "oci_media_types", oci_media_types)
        if tar is None:
            tar = True
        if tar is not None:
            pulumi.set(__self__, "tar", tar)

    @property
    @pulumi.getter
    def annotations(self) -> Optional[Mapping[str, str]]:
        """
        Attach an arbitrary key/value annotation to the image.
        """
        return pulumi.get(self, "annotations")

    @property
    @pulumi.getter
    def compression(self) -> Optional['CompressionType']:
        """
        The compression type to use.
        """
        return pulumi.get(self, "compression")

    @property
    @pulumi.getter(name="compressionLevel")
    def compression_level(self) -> Optional[int]:
        """
        Compression level from 0 to 22.
        """
        return pulumi.get(self, "compression_level")

    @property
    @pulumi.getter
    def dest(self) -> Optional[str]:
        """
        The local export path.
        """
        return pulumi.get(self, "dest")

    @property
    @pulumi.getter(name="forceCompression")
    def force_compression(self) -> Optional[bool]:
        """
        Forcefully apply compression.
        """
        return pulumi.get(self, "force_compression")

    @property
    @pulumi.getter
    def names(self) -> Optional[Sequence[str]]:
        """
        Specify images names to export. This is overridden if tags are already specified.
        """
        return pulumi.get(self, "names")

    @property
    @pulumi.getter(name="ociMediaTypes")
    def oci_media_types(self) -> Optional[bool]:
        """
        Use OCI media types in exporter manifests.
        """
        return pulumi.get(self, "oci_media_types")

    @property
    @pulumi.getter
    def tar(self) -> Optional[bool]:
        """
        Bundle the output into a tarball layout.
        """
        return pulumi.get(self, "tar")


@pulumi.output_type
class ExportRegistry(dict):
    @staticmethod
    def __key_warning(key: str):
        suggest = None
        if key == "compressionLevel":
            suggest = "compression_level"
        elif key == "danglingNamePrefix":
            suggest = "dangling_name_prefix"
        elif key == "forceCompression":
            suggest = "force_compression"
        elif key == "nameCanonical":
            suggest = "name_canonical"
        elif key == "ociMediaTypes":
            suggest = "oci_media_types"
        elif key == "pushByDigest":
            suggest = "push_by_digest"

        if suggest:
            pulumi.log.warn(f"Key '{key}' not found in ExportRegistry. Access the value via the '{suggest}' property getter instead.")

    def __getitem__(self, key: str) -> Any:
        ExportRegistry.__key_warning(key)
        return super().__getitem__(key)

    def get(self, key: str, default = None) -> Any:
        ExportRegistry.__key_warning(key)
        return super().get(key, default)

    def __init__(__self__, *,
                 annotations: Optional[Mapping[str, str]] = None,
                 compression: Optional['CompressionType'] = None,
                 compression_level: Optional[int] = None,
                 dangling_name_prefix: Optional[str] = None,
                 force_compression: Optional[bool] = None,
                 insecure: Optional[bool] = None,
                 name_canonical: Optional[bool] = None,
                 names: Optional[Sequence[str]] = None,
                 oci_media_types: Optional[bool] = None,
                 push: Optional[bool] = None,
                 push_by_digest: Optional[bool] = None,
                 store: Optional[bool] = None,
                 unpack: Optional[bool] = None):
        """
        :param Mapping[str, str] annotations: Attach an arbitrary key/value annotation to the image.
        :param 'CompressionType' compression: The compression type to use.
        :param int compression_level: Compression level from 0 to 22.
        :param str dangling_name_prefix: Name image with `prefix@<digest>`, used for anonymous images.
        :param bool force_compression: Forcefully apply compression.
        :param bool insecure: Allow pushing to an insecure registry.
        :param bool name_canonical: Add additional canonical name (`name@<digest>`).
        :param Sequence[str] names: Specify images names to export. This is overridden if tags are already specified.
        :param bool oci_media_types: Use OCI media types in exporter manifests.
        :param bool push: Push after creating the image.
        :param bool push_by_digest: Push image without name.
        :param bool store: Store resulting images to the worker's image store and ensure all of
               its blobs are in the content store. 
               
               Defaults to `true`.
               
               Ignored if the worker doesn't have image store (when using OCI workers,
               for example).
        :param bool unpack: Unpack image after creation (for use with containerd). Defaults to
               `false`.
        """
        if annotations is not None:
            pulumi.set(__self__, "annotations", annotations)
        if compression is None:
            compression = 'gzip'
        if compression is not None:
            pulumi.set(__self__, "compression", compression)
        if compression_level is None:
            compression_level = 0
        if compression_level is not None:
            pulumi.set(__self__, "compression_level", compression_level)
        if dangling_name_prefix is not None:
            pulumi.set(__self__, "dangling_name_prefix", dangling_name_prefix)
        if force_compression is None:
            force_compression = False
        if force_compression is not None:
            pulumi.set(__self__, "force_compression", force_compression)
        if insecure is not None:
            pulumi.set(__self__, "insecure", insecure)
        if name_canonical is not None:
            pulumi.set(__self__, "name_canonical", name_canonical)
        if names is not None:
            pulumi.set(__self__, "names", names)
        if oci_media_types is None:
            oci_media_types = False
        if oci_media_types is not None:
            pulumi.set(__self__, "oci_media_types", oci_media_types)
        if push is None:
            push = True
        if push is not None:
            pulumi.set(__self__, "push", push)
        if push_by_digest is not None:
            pulumi.set(__self__, "push_by_digest", push_by_digest)
        if store is None:
            store = True
        if store is not None:
            pulumi.set(__self__, "store", store)
        if unpack is not None:
            pulumi.set(__self__, "unpack", unpack)

    @property
    @pulumi.getter
    def annotations(self) -> Optional[Mapping[str, str]]:
        """
        Attach an arbitrary key/value annotation to the image.
        """
        return pulumi.get(self, "annotations")

    @property
    @pulumi.getter
    def compression(self) -> Optional['CompressionType']:
        """
        The compression type to use.
        """
        return pulumi.get(self, "compression")

    @property
    @pulumi.getter(name="compressionLevel")
    def compression_level(self) -> Optional[int]:
        """
        Compression level from 0 to 22.
        """
        return pulumi.get(self, "compression_level")

    @property
    @pulumi.getter(name="danglingNamePrefix")
    def dangling_name_prefix(self) -> Optional[str]:
        """
        Name image with `prefix@<digest>`, used for anonymous images.
        """
        return pulumi.get(self, "dangling_name_prefix")

    @property
    @pulumi.getter(name="forceCompression")
    def force_compression(self) -> Optional[bool]:
        """
        Forcefully apply compression.
        """
        return pulumi.get(self, "force_compression")

    @property
    @pulumi.getter
    def insecure(self) -> Optional[bool]:
        """
        Allow pushing to an insecure registry.
        """
        return pulumi.get(self, "insecure")

    @property
    @pulumi.getter(name="nameCanonical")
    def name_canonical(self) -> Optional[bool]:
        """
        Add additional canonical name (`name@<digest>`).
        """
        return pulumi.get(self, "name_canonical")

    @property
    @pulumi.getter
    def names(self) -> Optional[Sequence[str]]:
        """
        Specify images names to export. This is overridden if tags are already specified.
        """
        return pulumi.get(self, "names")

    @property
    @pulumi.getter(name="ociMediaTypes")
    def oci_media_types(self) -> Optional[bool]:
        """
        Use OCI media types in exporter manifests.
        """
        return pulumi.get(self, "oci_media_types")

    @property
    @pulumi.getter
    def push(self) -> Optional[bool]:
        """
        Push after creating the image.
        """
        return pulumi.get(self, "push")

    @property
    @pulumi.getter(name="pushByDigest")
    def push_by_digest(self) -> Optional[bool]:
        """
        Push image without name.
        """
        return pulumi.get(self, "push_by_digest")

    @property
    @pulumi.getter
    def store(self) -> Optional[bool]:
        """
        Store resulting images to the worker's image store and ensure all of
        its blobs are in the content store. 

        Defaults to `true`.

        Ignored if the worker doesn't have image store (when using OCI workers,
        for example).
        """
        return pulumi.get(self, "store")

    @property
    @pulumi.getter
    def unpack(self) -> Optional[bool]:
        """
        Unpack image after creation (for use with containerd). Defaults to
        `false`.
        """
        return pulumi.get(self, "unpack")


@pulumi.output_type
class ExportTar(dict):
    def __init__(__self__, *,
                 dest: str):
        """
        :param str dest: Output path.
        """
        pulumi.set(__self__, "dest", dest)

    @property
    @pulumi.getter
    def dest(self) -> str:
        """
        Output path.
        """
        return pulumi.get(self, "dest")


@pulumi.output_type
class Manifest(dict):
    def __init__(__self__, *,
                 digest: str,
                 platform: 'outputs.ManifestPlatform',
                 ref: str,
                 size: int):
        """
        :param str digest: The SHA256 digest of the manifest.
        :param 'ManifestPlatformArgs' platform: The manifest's platform.
        :param str ref: The manifest's canonical ref.
        :param int size: The size of the manifest in bytes.
        """
        pulumi.set(__self__, "digest", digest)
        pulumi.set(__self__, "platform", platform)
        pulumi.set(__self__, "ref", ref)
        pulumi.set(__self__, "size", size)

    @property
    @pulumi.getter
    def digest(self) -> str:
        """
        The SHA256 digest of the manifest.
        """
        return pulumi.get(self, "digest")

    @property
    @pulumi.getter
    def platform(self) -> 'outputs.ManifestPlatform':
        """
        The manifest's platform.
        """
        return pulumi.get(self, "platform")

    @property
    @pulumi.getter
    def ref(self) -> str:
        """
        The manifest's canonical ref.
        """
        return pulumi.get(self, "ref")

    @property
    @pulumi.getter
    def size(self) -> int:
        """
        The size of the manifest in bytes.
        """
        return pulumi.get(self, "size")


@pulumi.output_type
class ManifestPlatform(dict):
    def __init__(__self__, *,
                 architecture: str,
                 os: str):
        """
        :param str architecture: The manifest's architecture.
        :param str os: The manifest's operating systen.
        """
        pulumi.set(__self__, "architecture", architecture)
        pulumi.set(__self__, "os", os)

    @property
    @pulumi.getter
    def architecture(self) -> str:
        """
        The manifest's architecture.
        """
        return pulumi.get(self, "architecture")

    @property
    @pulumi.getter
    def os(self) -> str:
        """
        The manifest's operating systen.
        """
        return pulumi.get(self, "os")


@pulumi.output_type
class RegistryAuth(dict):
    def __init__(__self__, *,
                 address: str,
                 password: Optional[str] = None,
                 username: Optional[str] = None):
        """
        :param str address: The registry's address (e.g. "docker.io").
        :param str password: Password or token for the registry.
        :param str username: Username for the registry.
        """
        pulumi.set(__self__, "address", address)
        if password is not None:
            pulumi.set(__self__, "password", password)
        if username is not None:
            pulumi.set(__self__, "username", username)

    @property
    @pulumi.getter
    def address(self) -> str:
        """
        The registry's address (e.g. "docker.io").
        """
        return pulumi.get(self, "address")

    @property
    @pulumi.getter
    def password(self) -> Optional[str]:
        """
        Password or token for the registry.
        """
        return pulumi.get(self, "password")

    @property
    @pulumi.getter
    def username(self) -> Optional[str]:
        """
        Username for the registry.
        """
        return pulumi.get(self, "username")





# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from .. import _utilities
from . import outputs
from ._enums import *
from ._inputs import *

__all__ = ['ImageArgs', 'Image']

@pulumi.input_type
class ImageArgs:
    def __init__(__self__, *,
                 build_args: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
                 build_on_preview: Optional[pulumi.Input[bool]] = None,
                 builder: Optional[pulumi.Input['BuilderConfigArgs']] = None,
                 cache_from: Optional[pulumi.Input[Sequence[pulumi.Input['CacheFromEntryArgs']]]] = None,
                 cache_to: Optional[pulumi.Input[Sequence[pulumi.Input['CacheToEntryArgs']]]] = None,
                 context: Optional[pulumi.Input['BuildContextArgs']] = None,
                 dockerfile: Optional[pulumi.Input['DockerfileArgs']] = None,
                 exports: Optional[pulumi.Input[Sequence[pulumi.Input['ExportEntryArgs']]]] = None,
                 labels: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
                 platforms: Optional[pulumi.Input[Sequence[pulumi.Input['Platform']]]] = None,
                 pull: Optional[pulumi.Input[bool]] = None,
                 registries: Optional[pulumi.Input[Sequence[pulumi.Input['RegistryAuthArgs']]]] = None,
                 secrets: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
                 tags: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 targets: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None):
        """
        The set of arguments for constructing a Image resource.
        :param pulumi.Input[Mapping[str, pulumi.Input[str]]] build_args: `ARG` names and values to set during the build.
               
               These variables are accessed like environment variables inside `RUN`
               instructions.
               
               Build arguments are persisted in the image, so you should use `secrets`
               if these arguments are sensitive.
        :param pulumi.Input[bool] build_on_preview: When `true`, attempt to build the image during previews. The image will
               not be pushed to registries, however caches will still populated.
        :param pulumi.Input['BuilderConfigArgs'] builder: Builder configuration.
        :param pulumi.Input[Sequence[pulumi.Input['CacheFromEntryArgs']]] cache_from: External cache configuration.
        :param pulumi.Input[Sequence[pulumi.Input['CacheToEntryArgs']]] cache_to: Cache export configuration.
        :param pulumi.Input['BuildContextArgs'] context: Build context settings.
        :param pulumi.Input['DockerfileArgs'] dockerfile: Dockerfile settings.
        :param pulumi.Input[Sequence[pulumi.Input['ExportEntryArgs']]] exports: Controls where images are persisted after building.
               
               Images are only stored in the local cache unless `exports` are
               explicitly configured.
        :param pulumi.Input[Mapping[str, pulumi.Input[str]]] labels: Attach arbitrary key/value metadata to the image.
        :param pulumi.Input[Sequence[pulumi.Input['Platform']]] platforms: Set target platform(s) for the build. Defaults to the host's platform
        :param pulumi.Input[bool] pull: Always pull referenced images.
        :param pulumi.Input[Sequence[pulumi.Input['RegistryAuthArgs']]] registries: Registry credentials. Required if reading or exporting to private
               repositories.
        :param pulumi.Input[Mapping[str, pulumi.Input[str]]] secrets: A mapping of secret names to their corresponding values.
               
               Unlike the Docker CLI, these can be passed by value and do not need to
               exist on-disk or in environment variables.
               
               Build arguments and environment variables are persistent in the final
               image, so you should use this for sensitive values.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] tags: Name and optionally a tag (format: `name:tag`).
               
               If exporting to a registry, the name should include the fully qualified
               registry address (e.g. `docker.io/pulumi/pulumi:latest`).
        :param pulumi.Input[Sequence[pulumi.Input[str]]] targets: Set the target build stage(s) to build.
               
               If not specified all targets will be built by default.
        """
        if build_args is not None:
            pulumi.set(__self__, "build_args", build_args)
        if build_on_preview is not None:
            pulumi.set(__self__, "build_on_preview", build_on_preview)
        if builder is not None:
            pulumi.set(__self__, "builder", builder)
        if cache_from is not None:
            pulumi.set(__self__, "cache_from", cache_from)
        if cache_to is not None:
            pulumi.set(__self__, "cache_to", cache_to)
        if context is not None:
            pulumi.set(__self__, "context", context)
        if dockerfile is not None:
            pulumi.set(__self__, "dockerfile", dockerfile)
        if exports is not None:
            pulumi.set(__self__, "exports", exports)
        if labels is not None:
            pulumi.set(__self__, "labels", labels)
        if platforms is not None:
            pulumi.set(__self__, "platforms", platforms)
        if pull is not None:
            pulumi.set(__self__, "pull", pull)
        if registries is not None:
            pulumi.set(__self__, "registries", registries)
        if secrets is not None:
            pulumi.set(__self__, "secrets", secrets)
        if tags is not None:
            pulumi.set(__self__, "tags", tags)
        if targets is not None:
            pulumi.set(__self__, "targets", targets)

    @property
    @pulumi.getter(name="buildArgs")
    def build_args(self) -> Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]]:
        """
        `ARG` names and values to set during the build.

        These variables are accessed like environment variables inside `RUN`
        instructions.

        Build arguments are persisted in the image, so you should use `secrets`
        if these arguments are sensitive.
        """
        return pulumi.get(self, "build_args")

    @build_args.setter
    def build_args(self, value: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]]):
        pulumi.set(self, "build_args", value)

    @property
    @pulumi.getter(name="buildOnPreview")
    def build_on_preview(self) -> Optional[pulumi.Input[bool]]:
        """
        When `true`, attempt to build the image during previews. The image will
        not be pushed to registries, however caches will still populated.
        """
        return pulumi.get(self, "build_on_preview")

    @build_on_preview.setter
    def build_on_preview(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "build_on_preview", value)

    @property
    @pulumi.getter
    def builder(self) -> Optional[pulumi.Input['BuilderConfigArgs']]:
        """
        Builder configuration.
        """
        return pulumi.get(self, "builder")

    @builder.setter
    def builder(self, value: Optional[pulumi.Input['BuilderConfigArgs']]):
        pulumi.set(self, "builder", value)

    @property
    @pulumi.getter(name="cacheFrom")
    def cache_from(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['CacheFromEntryArgs']]]]:
        """
        External cache configuration.
        """
        return pulumi.get(self, "cache_from")

    @cache_from.setter
    def cache_from(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['CacheFromEntryArgs']]]]):
        pulumi.set(self, "cache_from", value)

    @property
    @pulumi.getter(name="cacheTo")
    def cache_to(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['CacheToEntryArgs']]]]:
        """
        Cache export configuration.
        """
        return pulumi.get(self, "cache_to")

    @cache_to.setter
    def cache_to(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['CacheToEntryArgs']]]]):
        pulumi.set(self, "cache_to", value)

    @property
    @pulumi.getter
    def context(self) -> Optional[pulumi.Input['BuildContextArgs']]:
        """
        Build context settings.
        """
        return pulumi.get(self, "context")

    @context.setter
    def context(self, value: Optional[pulumi.Input['BuildContextArgs']]):
        pulumi.set(self, "context", value)

    @property
    @pulumi.getter
    def dockerfile(self) -> Optional[pulumi.Input['DockerfileArgs']]:
        """
        Dockerfile settings.
        """
        return pulumi.get(self, "dockerfile")

    @dockerfile.setter
    def dockerfile(self, value: Optional[pulumi.Input['DockerfileArgs']]):
        pulumi.set(self, "dockerfile", value)

    @property
    @pulumi.getter
    def exports(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['ExportEntryArgs']]]]:
        """
        Controls where images are persisted after building.

        Images are only stored in the local cache unless `exports` are
        explicitly configured.
        """
        return pulumi.get(self, "exports")

    @exports.setter
    def exports(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['ExportEntryArgs']]]]):
        pulumi.set(self, "exports", value)

    @property
    @pulumi.getter
    def labels(self) -> Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]]:
        """
        Attach arbitrary key/value metadata to the image.
        """
        return pulumi.get(self, "labels")

    @labels.setter
    def labels(self, value: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]]):
        pulumi.set(self, "labels", value)

    @property
    @pulumi.getter
    def platforms(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['Platform']]]]:
        """
        Set target platform(s) for the build. Defaults to the host's platform
        """
        return pulumi.get(self, "platforms")

    @platforms.setter
    def platforms(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['Platform']]]]):
        pulumi.set(self, "platforms", value)

    @property
    @pulumi.getter
    def pull(self) -> Optional[pulumi.Input[bool]]:
        """
        Always pull referenced images.
        """
        return pulumi.get(self, "pull")

    @pull.setter
    def pull(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "pull", value)

    @property
    @pulumi.getter
    def registries(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['RegistryAuthArgs']]]]:
        """
        Registry credentials. Required if reading or exporting to private
        repositories.
        """
        return pulumi.get(self, "registries")

    @registries.setter
    def registries(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['RegistryAuthArgs']]]]):
        pulumi.set(self, "registries", value)

    @property
    @pulumi.getter
    def secrets(self) -> Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]]:
        """
        A mapping of secret names to their corresponding values.

        Unlike the Docker CLI, these can be passed by value and do not need to
        exist on-disk or in environment variables.

        Build arguments and environment variables are persistent in the final
        image, so you should use this for sensitive values.
        """
        return pulumi.get(self, "secrets")

    @secrets.setter
    def secrets(self, value: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]]):
        pulumi.set(self, "secrets", value)

    @property
    @pulumi.getter
    def tags(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]:
        """
        Name and optionally a tag (format: `name:tag`).

        If exporting to a registry, the name should include the fully qualified
        registry address (e.g. `docker.io/pulumi/pulumi:latest`).
        """
        return pulumi.get(self, "tags")

    @tags.setter
    def tags(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]):
        pulumi.set(self, "tags", value)

    @property
    @pulumi.getter
    def targets(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]:
        """
        Set the target build stage(s) to build.

        If not specified all targets will be built by default.
        """
        return pulumi.get(self, "targets")

    @targets.setter
    def targets(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]):
        pulumi.set(self, "targets", value)


class Image(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 build_args: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
                 build_on_preview: Optional[pulumi.Input[bool]] = None,
                 builder: Optional[pulumi.Input[pulumi.InputType['BuilderConfigArgs']]] = None,
                 cache_from: Optional[pulumi.Input[Sequence[pulumi.Input[pulumi.InputType['CacheFromEntryArgs']]]]] = None,
                 cache_to: Optional[pulumi.Input[Sequence[pulumi.Input[pulumi.InputType['CacheToEntryArgs']]]]] = None,
                 context: Optional[pulumi.Input[pulumi.InputType['BuildContextArgs']]] = None,
                 dockerfile: Optional[pulumi.Input[pulumi.InputType['DockerfileArgs']]] = None,
                 exports: Optional[pulumi.Input[Sequence[pulumi.Input[pulumi.InputType['ExportEntryArgs']]]]] = None,
                 labels: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
                 platforms: Optional[pulumi.Input[Sequence[pulumi.Input['Platform']]]] = None,
                 pull: Optional[pulumi.Input[bool]] = None,
                 registries: Optional[pulumi.Input[Sequence[pulumi.Input[pulumi.InputType['RegistryAuthArgs']]]]] = None,
                 secrets: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
                 tags: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 targets: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 __props__=None):
        """
        A Docker image built using buildx -- Docker's interface to the improved
        BuildKit backend.

        **This resource is experimental and subject to change.**

        API types are unstable. Subsequent releases _may_ require manual edits
        to your state file(s) in order to adopt API changes.

        Only use this resource if you understand and accept the risks.

        ## Example Usage
        ### Multi-platform image
        ```python
        import pulumi
        import pulumi_docker as docker

        image = docker.buildx.Image("image",
            context=docker.buildx.BuildContextArgs(
                location="app",
            ),
            dockerfile=docker.buildx.DockerfileArgs(
                location="app/Dockerfile",
            ),
            platforms=[
                docker.buildx/image.Platform.PLAN9_AMD64,
                docker.buildx/image.Platform.PLAN9_386,
            ])
        ```
        ### Registry export
        ```python
        import pulumi
        import pulumi_docker as docker

        image = docker.buildx.Image("image",
            context=docker.buildx.BuildContextArgs(
                location="app",
            ),
            dockerfile=docker.buildx.DockerfileArgs(
                location="app/Dockerfile",
            ),
            exports=[docker.buildx.ExportEntryArgs(
                registry=docker.buildx.ExportRegistryArgs(
                    oci_media_types=True,
                ),
            )],
            registries=[docker.buildx.RegistryAuthArgs(
                address="docker.io",
                password=docker_hub_password,
                username="pulumibot",
            )])
        ```
        ### Caching
        ```python
        import pulumi
        import pulumi_docker as docker

        image = docker.buildx.Image("image",
            cache_from=[docker.buildx.CacheFromEntryArgs(
                local=docker.buildx.CacheFromLocalArgs(
                    src="tmp/cache",
                ),
            )],
            cache_to=[docker.buildx.CacheToEntryArgs(
                local=docker.buildx.CacheToLocalArgs(
                    dest="tmp/cache",
                    mode=docker.buildx/image.CacheMode.MAX,
                ),
            )],
            context=docker.buildx.BuildContextArgs(
                location="app",
            ),
            dockerfile=docker.buildx.DockerfileArgs(
                location="app/Dockerfile",
            ))
        ```
        ### Build arguments
        ```python
        import pulumi
        import pulumi_docker as docker

        image = docker.buildx.Image("image",
            build_args={
                "SET_ME_TO_TRUE": "true",
            },
            context=docker.buildx.BuildContextArgs(
                location="app",
            ),
            dockerfile=docker.buildx.DockerfileArgs(
                location="app/Dockerfile",
            ))
        ```
        ### Build targets
        ```python
        import pulumi
        import pulumi_docker as docker

        image = docker.buildx.Image("image",
            context=docker.buildx.BuildContextArgs(
                location="app",
            ),
            dockerfile=docker.buildx.DockerfileArgs(
                location="app/Dockerfile",
            ),
            targets=[
                "build-me",
                "also-build-me",
            ])
        ```
        ### Named contexts
        ```python
        import pulumi
        import pulumi_docker as docker

        image = docker.buildx.Image("image",
            context=docker.buildx.BuildContextArgs(
                location="app",
                named={
                    "golang:latest": docker.buildx.ContextArgs(
                        location="docker-image://golang@sha256:b8e62cf593cdaff36efd90aa3a37de268e6781a2e68c6610940c48f7cdf36984",
                    ),
                },
            ),
            dockerfile=docker.buildx.DockerfileArgs(
                location="app/Dockerfile",
            ))
        ```
        ### Remote context
        ```python
        import pulumi
        import pulumi_docker as docker

        image = docker.buildx.Image("image", context=docker.buildx.BuildContextArgs(
            location="https://raw.githubusercontent.com/pulumi/pulumi-docker/api-types/provider/testdata/Dockerfile",
        ))
        ```
        ### Inline Dockerfile
        ```python
        import pulumi
        import pulumi_docker as docker

        image = docker.buildx.Image("image",
            context=docker.buildx.BuildContextArgs(
                location="app",
            ),
            dockerfile=docker.buildx.DockerfileArgs(
                inline=\"\"\"FROM busybox
        COPY hello.c ./
        \"\"\",
            ))
        ```
        ### Remote context
        ```python
        import pulumi
        import pulumi_docker as docker

        image = docker.buildx.Image("image",
            context=docker.buildx.BuildContextArgs(
                location="https://github.com/docker-library/hello-world.git",
            ),
            dockerfile=docker.buildx.DockerfileArgs(
                location="app/Dockerfile",
            ))
        ```
        ### Local export
        ```python
        import pulumi
        import pulumi_docker as docker

        image = docker.buildx.Image("image",
            context=docker.buildx.BuildContextArgs(
                location="app",
            ),
            dockerfile=docker.buildx.DockerfileArgs(
                location="app/Dockerfile",
            ),
            exports=[docker.buildx.ExportEntryArgs(
                docker=docker.buildx.ExportDockerArgs(
                    tar=True,
                ),
            )])
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[Mapping[str, pulumi.Input[str]]] build_args: `ARG` names and values to set during the build.
               
               These variables are accessed like environment variables inside `RUN`
               instructions.
               
               Build arguments are persisted in the image, so you should use `secrets`
               if these arguments are sensitive.
        :param pulumi.Input[bool] build_on_preview: When `true`, attempt to build the image during previews. The image will
               not be pushed to registries, however caches will still populated.
        :param pulumi.Input[pulumi.InputType['BuilderConfigArgs']] builder: Builder configuration.
        :param pulumi.Input[Sequence[pulumi.Input[pulumi.InputType['CacheFromEntryArgs']]]] cache_from: External cache configuration.
        :param pulumi.Input[Sequence[pulumi.Input[pulumi.InputType['CacheToEntryArgs']]]] cache_to: Cache export configuration.
        :param pulumi.Input[pulumi.InputType['BuildContextArgs']] context: Build context settings.
        :param pulumi.Input[pulumi.InputType['DockerfileArgs']] dockerfile: Dockerfile settings.
        :param pulumi.Input[Sequence[pulumi.Input[pulumi.InputType['ExportEntryArgs']]]] exports: Controls where images are persisted after building.
               
               Images are only stored in the local cache unless `exports` are
               explicitly configured.
        :param pulumi.Input[Mapping[str, pulumi.Input[str]]] labels: Attach arbitrary key/value metadata to the image.
        :param pulumi.Input[Sequence[pulumi.Input['Platform']]] platforms: Set target platform(s) for the build. Defaults to the host's platform
        :param pulumi.Input[bool] pull: Always pull referenced images.
        :param pulumi.Input[Sequence[pulumi.Input[pulumi.InputType['RegistryAuthArgs']]]] registries: Registry credentials. Required if reading or exporting to private
               repositories.
        :param pulumi.Input[Mapping[str, pulumi.Input[str]]] secrets: A mapping of secret names to their corresponding values.
               
               Unlike the Docker CLI, these can be passed by value and do not need to
               exist on-disk or in environment variables.
               
               Build arguments and environment variables are persistent in the final
               image, so you should use this for sensitive values.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] tags: Name and optionally a tag (format: `name:tag`).
               
               If exporting to a registry, the name should include the fully qualified
               registry address (e.g. `docker.io/pulumi/pulumi:latest`).
        :param pulumi.Input[Sequence[pulumi.Input[str]]] targets: Set the target build stage(s) to build.
               
               If not specified all targets will be built by default.
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: Optional[ImageArgs] = None,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        A Docker image built using buildx -- Docker's interface to the improved
        BuildKit backend.

        **This resource is experimental and subject to change.**

        API types are unstable. Subsequent releases _may_ require manual edits
        to your state file(s) in order to adopt API changes.

        Only use this resource if you understand and accept the risks.

        ## Example Usage
        ### Multi-platform image
        ```python
        import pulumi
        import pulumi_docker as docker

        image = docker.buildx.Image("image",
            context=docker.buildx.BuildContextArgs(
                location="app",
            ),
            dockerfile=docker.buildx.DockerfileArgs(
                location="app/Dockerfile",
            ),
            platforms=[
                docker.buildx/image.Platform.PLAN9_AMD64,
                docker.buildx/image.Platform.PLAN9_386,
            ])
        ```
        ### Registry export
        ```python
        import pulumi
        import pulumi_docker as docker

        image = docker.buildx.Image("image",
            context=docker.buildx.BuildContextArgs(
                location="app",
            ),
            dockerfile=docker.buildx.DockerfileArgs(
                location="app/Dockerfile",
            ),
            exports=[docker.buildx.ExportEntryArgs(
                registry=docker.buildx.ExportRegistryArgs(
                    oci_media_types=True,
                ),
            )],
            registries=[docker.buildx.RegistryAuthArgs(
                address="docker.io",
                password=docker_hub_password,
                username="pulumibot",
            )])
        ```
        ### Caching
        ```python
        import pulumi
        import pulumi_docker as docker

        image = docker.buildx.Image("image",
            cache_from=[docker.buildx.CacheFromEntryArgs(
                local=docker.buildx.CacheFromLocalArgs(
                    src="tmp/cache",
                ),
            )],
            cache_to=[docker.buildx.CacheToEntryArgs(
                local=docker.buildx.CacheToLocalArgs(
                    dest="tmp/cache",
                    mode=docker.buildx/image.CacheMode.MAX,
                ),
            )],
            context=docker.buildx.BuildContextArgs(
                location="app",
            ),
            dockerfile=docker.buildx.DockerfileArgs(
                location="app/Dockerfile",
            ))
        ```
        ### Build arguments
        ```python
        import pulumi
        import pulumi_docker as docker

        image = docker.buildx.Image("image",
            build_args={
                "SET_ME_TO_TRUE": "true",
            },
            context=docker.buildx.BuildContextArgs(
                location="app",
            ),
            dockerfile=docker.buildx.DockerfileArgs(
                location="app/Dockerfile",
            ))
        ```
        ### Build targets
        ```python
        import pulumi
        import pulumi_docker as docker

        image = docker.buildx.Image("image",
            context=docker.buildx.BuildContextArgs(
                location="app",
            ),
            dockerfile=docker.buildx.DockerfileArgs(
                location="app/Dockerfile",
            ),
            targets=[
                "build-me",
                "also-build-me",
            ])
        ```
        ### Named contexts
        ```python
        import pulumi
        import pulumi_docker as docker

        image = docker.buildx.Image("image",
            context=docker.buildx.BuildContextArgs(
                location="app",
                named={
                    "golang:latest": docker.buildx.ContextArgs(
                        location="docker-image://golang@sha256:b8e62cf593cdaff36efd90aa3a37de268e6781a2e68c6610940c48f7cdf36984",
                    ),
                },
            ),
            dockerfile=docker.buildx.DockerfileArgs(
                location="app/Dockerfile",
            ))
        ```
        ### Remote context
        ```python
        import pulumi
        import pulumi_docker as docker

        image = docker.buildx.Image("image", context=docker.buildx.BuildContextArgs(
            location="https://raw.githubusercontent.com/pulumi/pulumi-docker/api-types/provider/testdata/Dockerfile",
        ))
        ```
        ### Inline Dockerfile
        ```python
        import pulumi
        import pulumi_docker as docker

        image = docker.buildx.Image("image",
            context=docker.buildx.BuildContextArgs(
                location="app",
            ),
            dockerfile=docker.buildx.DockerfileArgs(
                inline=\"\"\"FROM busybox
        COPY hello.c ./
        \"\"\",
            ))
        ```
        ### Remote context
        ```python
        import pulumi
        import pulumi_docker as docker

        image = docker.buildx.Image("image",
            context=docker.buildx.BuildContextArgs(
                location="https://github.com/docker-library/hello-world.git",
            ),
            dockerfile=docker.buildx.DockerfileArgs(
                location="app/Dockerfile",
            ))
        ```
        ### Local export
        ```python
        import pulumi
        import pulumi_docker as docker

        image = docker.buildx.Image("image",
            context=docker.buildx.BuildContextArgs(
                location="app",
            ),
            dockerfile=docker.buildx.DockerfileArgs(
                location="app/Dockerfile",
            ),
            exports=[docker.buildx.ExportEntryArgs(
                docker=docker.buildx.ExportDockerArgs(
                    tar=True,
                ),
            )])
        ```

        :param str resource_name: The name of the resource.
        :param ImageArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(ImageArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 build_args: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
                 build_on_preview: Optional[pulumi.Input[bool]] = None,
                 builder: Optional[pulumi.Input[pulumi.InputType['BuilderConfigArgs']]] = None,
                 cache_from: Optional[pulumi.Input[Sequence[pulumi.Input[pulumi.InputType['CacheFromEntryArgs']]]]] = None,
                 cache_to: Optional[pulumi.Input[Sequence[pulumi.Input[pulumi.InputType['CacheToEntryArgs']]]]] = None,
                 context: Optional[pulumi.Input[pulumi.InputType['BuildContextArgs']]] = None,
                 dockerfile: Optional[pulumi.Input[pulumi.InputType['DockerfileArgs']]] = None,
                 exports: Optional[pulumi.Input[Sequence[pulumi.Input[pulumi.InputType['ExportEntryArgs']]]]] = None,
                 labels: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
                 platforms: Optional[pulumi.Input[Sequence[pulumi.Input['Platform']]]] = None,
                 pull: Optional[pulumi.Input[bool]] = None,
                 registries: Optional[pulumi.Input[Sequence[pulumi.Input[pulumi.InputType['RegistryAuthArgs']]]]] = None,
                 secrets: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
                 tags: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 targets: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = ImageArgs.__new__(ImageArgs)

            __props__.__dict__["build_args"] = build_args
            __props__.__dict__["build_on_preview"] = build_on_preview
            __props__.__dict__["builder"] = builder
            __props__.__dict__["cache_from"] = cache_from
            __props__.__dict__["cache_to"] = cache_to
            __props__.__dict__["context"] = context
            __props__.__dict__["dockerfile"] = dockerfile
            __props__.__dict__["exports"] = exports
            __props__.__dict__["labels"] = labels
            __props__.__dict__["platforms"] = platforms
            __props__.__dict__["pull"] = pull
            __props__.__dict__["registries"] = registries
            __props__.__dict__["secrets"] = None if secrets is None else pulumi.Output.secret(secrets)
            __props__.__dict__["tags"] = tags
            __props__.__dict__["targets"] = targets
            __props__.__dict__["context_hash"] = None
            __props__.__dict__["digests"] = None
        secret_opts = pulumi.ResourceOptions(additional_secret_outputs=["secrets"])
        opts = pulumi.ResourceOptions.merge(opts, secret_opts)
        super(Image, __self__).__init__(
            'docker:buildx/image:Image',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None) -> 'Image':
        """
        Get an existing Image resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = ImageArgs.__new__(ImageArgs)

        __props__.__dict__["build_args"] = None
        __props__.__dict__["build_on_preview"] = None
        __props__.__dict__["builder"] = None
        __props__.__dict__["cache_from"] = None
        __props__.__dict__["cache_to"] = None
        __props__.__dict__["context"] = None
        __props__.__dict__["context_hash"] = None
        __props__.__dict__["digests"] = None
        __props__.__dict__["dockerfile"] = None
        __props__.__dict__["exports"] = None
        __props__.__dict__["labels"] = None
        __props__.__dict__["platforms"] = None
        __props__.__dict__["pull"] = None
        __props__.__dict__["registries"] = None
        __props__.__dict__["secrets"] = None
        __props__.__dict__["tags"] = None
        __props__.__dict__["targets"] = None
        return Image(resource_name, opts=opts, __props__=__props__)

    @property
    @pulumi.getter(name="buildArgs")
    def build_args(self) -> pulumi.Output[Optional[Mapping[str, str]]]:
        """
        `ARG` names and values to set during the build.

        These variables are accessed like environment variables inside `RUN`
        instructions.

        Build arguments are persisted in the image, so you should use `secrets`
        if these arguments are sensitive.
        """
        return pulumi.get(self, "build_args")

    @property
    @pulumi.getter(name="buildOnPreview")
    def build_on_preview(self) -> pulumi.Output[Optional[bool]]:
        """
        When `true`, attempt to build the image during previews. The image will
        not be pushed to registries, however caches will still populated.
        """
        return pulumi.get(self, "build_on_preview")

    @property
    @pulumi.getter
    def builder(self) -> pulumi.Output[Optional['outputs.BuilderConfig']]:
        """
        Builder configuration.
        """
        return pulumi.get(self, "builder")

    @property
    @pulumi.getter(name="cacheFrom")
    def cache_from(self) -> pulumi.Output[Optional[Sequence['outputs.CacheFromEntry']]]:
        """
        External cache configuration.
        """
        return pulumi.get(self, "cache_from")

    @property
    @pulumi.getter(name="cacheTo")
    def cache_to(self) -> pulumi.Output[Optional[Sequence['outputs.CacheToEntry']]]:
        """
        Cache export configuration.
        """
        return pulumi.get(self, "cache_to")

    @property
    @pulumi.getter
    def context(self) -> pulumi.Output[Optional['outputs.BuildContext']]:
        """
        Build context settings.
        """
        return pulumi.get(self, "context")

    @property
    @pulumi.getter(name="contextHash")
    def context_hash(self) -> pulumi.Output[Optional[str]]:
        """
        A preliminary hash of the image's build context.

        Pulumi uses this to determine if an image _may_ need to be re-built.
        """
        return pulumi.get(self, "context_hash")

    @property
    @pulumi.getter
    def digests(self) -> pulumi.Output[Optional[Mapping[str, Sequence[str]]]]:
        """
        A mapping of platform type to refs which were pushed to registries.
        """
        return pulumi.get(self, "digests")

    @property
    @pulumi.getter
    def dockerfile(self) -> pulumi.Output[Optional['outputs.Dockerfile']]:
        """
        Dockerfile settings.
        """
        return pulumi.get(self, "dockerfile")

    @property
    @pulumi.getter
    def exports(self) -> pulumi.Output[Optional[Sequence['outputs.ExportEntry']]]:
        """
        Controls where images are persisted after building.

        Images are only stored in the local cache unless `exports` are
        explicitly configured.
        """
        return pulumi.get(self, "exports")

    @property
    @pulumi.getter
    def labels(self) -> pulumi.Output[Optional[Mapping[str, str]]]:
        """
        Attach arbitrary key/value metadata to the image.
        """
        return pulumi.get(self, "labels")

    @property
    @pulumi.getter
    def platforms(self) -> pulumi.Output[Optional[Sequence['Platform']]]:
        """
        Set target platform(s) for the build. Defaults to the host's platform
        """
        return pulumi.get(self, "platforms")

    @property
    @pulumi.getter
    def pull(self) -> pulumi.Output[Optional[bool]]:
        """
        Always pull referenced images.
        """
        return pulumi.get(self, "pull")

    @property
    @pulumi.getter
    def registries(self) -> pulumi.Output[Optional[Sequence['outputs.RegistryAuth']]]:
        """
        Registry credentials. Required if reading or exporting to private
        repositories.
        """
        return pulumi.get(self, "registries")

    @property
    @pulumi.getter
    def secrets(self) -> pulumi.Output[Optional[Mapping[str, str]]]:
        """
        A mapping of secret names to their corresponding values.

        Unlike the Docker CLI, these can be passed by value and do not need to
        exist on-disk or in environment variables.

        Build arguments and environment variables are persistent in the final
        image, so you should use this for sensitive values.
        """
        return pulumi.get(self, "secrets")

    @property
    @pulumi.getter
    def tags(self) -> pulumi.Output[Optional[Sequence[str]]]:
        """
        Name and optionally a tag (format: `name:tag`).

        If exporting to a registry, the name should include the fully qualified
        registry address (e.g. `docker.io/pulumi/pulumi:latest`).
        """
        return pulumi.get(self, "tags")

    @property
    @pulumi.getter
    def targets(self) -> pulumi.Output[Optional[Sequence[str]]]:
        """
        Set the target build stage(s) to build.

        If not specified all targets will be built by default.
        """
        return pulumi.get(self, "targets")

